//console.log('CookiePlus', CookiePlus);

window.CookiePlusCookieBlocker = (function () {

  var newCookieObj = Object.getOwnPropertyDescriptor(Document.prototype, 'cookie') ||
    Object.getOwnPropertyDescriptor(HTMLDocument.prototype, 'cookie');
  var CP = CookiePlus;
  var CPAddon = CookiePlusAddonSettings;

  var app = {
  };

  app.cookie = {
    set: function (name, value, days, domain) {
      var expires = "";
      if (days) {
        days = parseFloat(days, 10);
        var date = new Date();
        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
        expires = "; expires=" + date.toGMTString();
      }
      var new_cookie = name + "=" + value + expires + "; path=/";
      if (domain) {
        new_cookie += ";domain=" + domain;
      }
      document.cookie = new_cookie;
    },
    get: function (name) {
      var nameEQ = name + "=";
      var ca = document.cookie.split(';');
      for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) === ' ') {
          c = c.substring(1, c.length);
        }
        if (c.indexOf(nameEQ) === 0) {
          return c.substring(nameEQ.length, c.length);
        }
      }
      return null;
    },
    remove: function (name, domain) {
      this.set(name, "", -1, domain);
    }
  };

  app.debug = app.cookie.get('cookieplus_debug') ? true : false,


  app.in_array = function (cookie_name, cookies_array) {
    var exists = false;
    for (var i = 0; i < cookies_array.length; i++) {
      if (cookies_array.indexOf(cookie_name) > -1) {
        exists = true;
        break;
      } else if (typeof cookies_array[i] == 'string' && cookies_array[i].slice(-1) == '*') {
        if (cookie_name.indexOf(cookies_array[i].slice(0, -1)) > -1) {
          exists = true;
          break;
        }
      }
    }
    return exists;
  };

  app.split_by_comma = function (value) {
    if (value) {
      value = value.replace(/(?:\r\n|\r|\n|\s)/g, '');
      return value.split(',');
    }
    return [];
  };

  app.array_diff = function (a1, a2) {
    return a1.concat(a2).filter(function (val, index, arr) {
      return arr.indexOf(val) === arr.lastIndexOf(val);
    });
  };

  app.array_unique = function (arr) {
    var result = [];
    arr.forEach(function (item) {
      if (item && result.indexOf(item) < 0) {
        result.push(item);
      }
    });
    return result;
  };

  app.get_allowed_groups = function () {
    var allowed_groups = '';
    var CookieConsentObj = app.get_cookie_obj();
    if (app.cookies_accepted() && CookieConsentObj.groups) {//Desde cookie
      allowed_groups = CookieConsentObj.groups;
    } else {//Desde plugin settings
      allowed_groups = 'group_1';//Grupo 1 siempre se permite
      for (var key in CP.CookieConsent.groups) {
        if (key != 'group_1' && CP.CookieConsent.groups.hasOwnProperty(key)) {
          if (CP.CookieConsent.groups[key].consent_given) {
            allowed_groups += ',' + key;
          }
        }
      }
    }
    return app.split_by_comma(allowed_groups);
  };

  app.get_blocked_cookies = function () {
    var allowed_groups = app.get_allowed_groups();
    var blocked_groups = app.array_diff(CP.CookieConsent.all_groups, allowed_groups);
    if (!blocked_groups) {
      return [];
    }
    var blocked_cookies = [];
    blocked_groups.forEach(function (group) {
      if (group != 'all' && group != 'group_1' && CP.CookieConsent.groups[group]) {
        blocked_cookies = blocked_cookies.concat(CP.CookieConsent.groups[group].blocked_cookies);
      }
    });
    return app.array_unique(blocked_cookies);
  };

  app.get_cookie_obj = function () {
    var CookieConsentValue = app.cookie.get(CP.CookieConsent.cookie_name);
    if (CookieConsentValue) {
      return JSON.parse(decodeURIComponent(CookieConsentValue));
    }
    return {};
  };

  app.cookies_accepted = function () {
    var CookieConsentObj = app.get_cookie_obj();
    if (CookieConsentObj) {
      if (CookieConsentObj.decline) {
        return false;
      } else if (CookieConsentObj.groups) {
        return true;
      }
    }
    return false;
  };


  if (CPAddon.disable_services) {
    //Disable Google Analytics
    if (CPAddon.disable_services.google_analytics && CPAddon.disable_services.google_analytics.disable) {
      window['ga-disable-' + CPAddon.disable_services.google_analytics.id] = true;
    }
  }

  if (CP.CookieConsent.enabled) {
    Object.defineProperty(document, 'cookie', {
      get: function () {
        return newCookieObj.get.call(document);
      },
      set: function (value) {
        cc('==== Set Cookie? : ', value);
        if (value && value.indexOf('=') > -1) {
          var pairs = value.split('=');
          var cookie_name = pairs[0].trim();

          //Si la cookie es una cookie del plugin, permitir crearla.
          var plugin_cookies = [CP.CookieConsent.cookie_name, CP.PrivacyPolicy.cookie_name, CP.TermsConditions.cookie_name];
          if (app.in_array(cookie_name, plugin_cookies)) {
            return newCookieObj.set.call(document, value);
          }

          var CookieConsentObj = app.get_cookie_obj();
          if (CP.CookieConsent.mode == 'block-all') {
            cc('= Mode', 'block-all');
            //Si ya aceptó las cookies, permitir crear cualquier cookie
            if (app.cookies_accepted()) {
              cc('Cookies Accepted, return');
              return newCookieObj.set.call(document, value);
            }
            //Si no, sólo permitir las cookies necesarias
            else if (app.in_array(cookie_name, CP.CookieConsent.allowed_cookies)) {
              cc('Set just CP.CookieConsent.allowed_cookies', cookie_name);
              return newCookieObj.set.call(document, value);
            } else {
              cc('Block cookie', cookie_name);
            }
          } else if (CP.CookieConsent.mode == 'by-groups') {
            cc('= Mode', 'by-groups');
            //Si ya aceptó las cookies, y aceptó todos los grupos, permitir todas las cookies
            if (app.cookies_accepted() && CookieConsentObj.groups == 'all') {
              cc('Cookies Accepted, return');
              return newCookieObj.set.call(document, value);
            }
            //Si aún no acepta cookies o no aceptó todos los grupos, bloquear las cookies de los grupos no aceptados
            else if (!app.in_array(cookie_name, app.get_blocked_cookies())) {
              cc('Set cookie by group:', cookie_name);
              return newCookieObj.set.call(document, value);
            } else {
              cc('Block cookie by group:', cookie_name);
            }
          } else {
            cc('= Cookies Not Accepted Yet');
          }
        }
        return value;
      },
    });
  } else {
    cc('CP.CookieConsent is disabled');
  }

  function c(msg) {
    console.log(msg);
  }

  function cc(msg, msg2) {
    if (app.debug) {
      if( msg2 ){
        console.log(msg,msg2);
      } else {
        console.log(msg);
      }
    }
  }


  return app;
})();