<?php namespace MasterPopupsAddonCookiePlus\Includes;

class CookieConsent {
    protected static $instance = null;
    public $plugin;
    private $system_cookies = array(
        'wp-settings*',
        'wordpress*',
    );

    public $option_id_cookie_name = 'cookie-consent-cookie-name';
    public $cookie_name = 'CookiePlusCookiesAccepted';

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    protected function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->system_cookies[] = session_name();
        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Debug
    |---------------------------------------------------------------------------------------------------
    */
    public function d( $var1, $var2 = null ){
        return $this->plugin->d( $var1, $var2 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        $wp_head_priority = Options::option( 'header-scripts-priority', 10 );
        $wp_footer_priority = Options::option( 'footer-scripts-priority', 10 );

        add_action( 'xbox_init', array( $this, 'on_init' ) );
        add_action( 'wp_head', array( $this, 'filter_integration_scripts' ), 1 );
        add_action( 'wp_head', array( $this, 'add_scripts_to_head' ), $wp_head_priority );
        add_action( 'wp_footer', array( $this, 'add_scripts_to_footer' ), $wp_footer_priority );
        add_action( 'wp_ajax_cookieplus_cookie_consent', array( $this, 'proccess_cookie_consent' ) );
        add_action( 'wp_ajax_nopriv_cookieplus_cookie_consent', array( $this, 'proccess_cookie_consent' ) );
        add_action( 'shutdown', array( $this, 'process_header_cookies' ), 0 );
        add_shortcode( 'cookieplus_block_content', array( $this, 'build_block_content_shortcode' ) );
        add_shortcode( 'cookieplus_btn_accept_cookies', array( $this, 'build_button_accept_cookies_shortcode' ) );
        add_shortcode( 'cookieplus_btn_decline_cookies', array( $this, 'build_button_decline_cookies_shortcode' ) );

    }

    /*
	|---------------------------------------------------------------------------------------------------
	| On init
	|---------------------------------------------------------------------------------------------------
	*/
    public function on_init(){
        if( $this->should_filter_page_content() ){
            ob_start( array( $this, 'filter_page_content' ) );
        }

        //          $currentCookieParams = session_get_cookie_params();
        ////        session_set_cookie_params(
        ////            $currentCookieParams["lifetime"],
        ////            $currentCookieParams["path"],
        ////            $currentCookieParams["domain"],
        ////            $currentCookieParams["secure"],
        ////            false//$currentCookieParams["httponly"]
        ////        );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si se debe filtrar el contenido
	|---------------------------------------------------------------------------------------------------
	*/
    public function should_filter_page_content(){
        if( is_admin() || Functions::is_login_page() || is_feed() || is_robots() || is_trackback() ) {
            return false;
        }

        //Si el consentimiento está desabilidado y el país actual no está dentro de la restricción
        if( ! $this->is_enabled() ){
            return false;
        }

        if( Options::option( 'content-blocker-enabled' ) == 'off' ){
            return false;
        }

        return true;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Filter page content
	|---------------------------------------------------------------------------------------------------
	*/
    public function filter_page_content( $buffer ){
        if( empty( $buffer) ) return $buffer;

        @ini_set( 'pcre.jit', false );//Avoid problems with PHP 7 PCRE's JIT compiler.

        $blocked_services = $this->get_blocked_services();
        $services_regex = $this->get_services_regex();
        $return = $buffer;

        foreach( $services_regex as $service => $regex ){
            if( $regex != '' ){
                $group_id = -1;
                foreach( $blocked_services as $g_id => $services ){
                    if( in_array( $service, $services ) ){
                        $group_id = $g_id;
                    }
                }
                if( $group_id != -1 && $this->is_group_enabled( $group_id ) ){
                    $allow_content = $this->allow_content_for_group( $group_id );
                    if( ! $allow_content ){
                        $return = preg_replace_callback( $regex, array( $this, 'filter_page_content_callback' ), $return );
                    }
                }
            }
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | filter_page_content_callback
    |---------------------------------------------------------------------------------------------------
    */
    public function filter_page_content_callback( $matches ){
        $message = Options::option( 'content-blocker-message' );
        $text = Options::option( 'content-blocker-button-text' );
        $button = do_shortcode( '[cookieplus_btn_accept_cookies]' . $text . '[/cookieplus_btn_accept_cookies]' );
        $message = "<div class='cookieplus-blocked-cookies-message'>$message</div>";
        $content = "<div class='cookieplus'>{$message}{$button}</div>";
        return $content;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Cookie Consent enabled
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_enabled(){
        $is_on = Options::option( 'cookie-consent-enabled' ) == 'on';
        $enabled_countries = $this->should_enabled_on_countries();
        return $is_on && $enabled_countries;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si se debe activar el consentimiento en determinados países
	|---------------------------------------------------------------------------------------------------
	*/
    public function should_enabled_on_countries(){
        if( ! class_exists( 'MasterPopups', false ) || ! class_exists( 'XboxItems', false ) ){
            return true;
        }
        $enabled = true;
        $activate_all_countries = Options::option( 'cookieplus-activate-all-countries' );
        $disable_all_countries = Options::option( 'cookieplus-disable-all-countries' );
        $geo_info = $this->plugin->get_geo_info();
        if( ! $geo_info ){
            return $enabled;
        }

        //Activate Countries
        $activate_countries = (array) Options::option( 'cookieplus-activate-countries' );
        if( Options::option( 'cookieplus-activate-on-eu-countries' ) == 'on' ){
            $activate_countries = array_merge( $activate_countries, array_keys( \XboxItems::eu_countries() ) );
        }
        if( in_array( $geo_info->country->isoCode, $activate_countries ) ){
            $enabled = true;
        } else{
            $enabled = false;
        }
        if( $activate_all_countries == 'on' ){
            $enabled = true;
        }

        //Disable Countries
        $disable_countries = (array) Options::option( 'cookieplus-disable-countries' );
        if( Options::option( 'cookieplus-disable-on-eu-countries' ) == 'on' ){
            $disable_countries = array_merge( $disable_countries, array_keys( \XboxItems::eu_countries() ) );
        }
        if( ! Functions::is_empty( $disable_countries ) && in_array( $geo_info->country->isoCode, $disable_countries ) ){
            $enabled = false;
        }
        if( $disable_all_countries == 'on' ){
            $enabled = false;
        }
        return $enabled;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Countries info
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_countries_info(){
        $geo_info = $this->plugin->get_geo_info();
        $current_country_iso_code = null;
        $current_country_name = null;
        if( $geo_info ){
            $current_country_iso_code = $geo_info->country->isoCode;
            $current_country_name = $geo_info->country->name;
        }
        return array(
            'current_country_iso_code' => $current_country_iso_code,
            'current_country_name' => $current_country_name,
            'cookie_bloquer' => array(
                'activate' => array(
                    'all_countries' => Options::option( 'cookieplus-activate-all-countries' ),
                    'countries' => (array) Options::option( 'cookieplus-activate-countries' ),
                    'eu_countries' => Options::option( 'cookieplus-activate-on-eu-countries' ),
                ),
                'disable' => array(
                    'all_countries' => Options::option( 'cookieplus-disable-all-countries' ),
                    'countries' => (array) Options::option( 'cookieplus-disable-countries' ),
                    'eu_countries' => Options::option( 'cookieplus-disable-on-eu-countries' ),
                )
            )
        );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Cookie Consent cookie name
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_cookie_name(){
        $cookie_name = Options::option( $this->option_id_cookie_name );
        $cookie_name = $cookie_name ? $cookie_name : $this->cookie_name;
        //No se permiten espacios en blanco
        //Php convierte puntos (.) en _ guiones bajos a los nombres de las cookies
        $cookie_name = str_replace( array( ' ', '.', ), array( '', '_' ), $cookie_name );
        $this->cookie_name = $cookie_name;
        return $this->cookie_name;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Comprueba si la cookie existe en el navegador y si ha aceptado o rechazado
	|---------------------------------------------------------------------------------------------------
	*/
    public function cookies_accepted_or_declined(){
        return $this->cookies_accepted() || $this->cookies_declined();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Comprueba si la cookie existe en el navegador y ha aceptado las cookies
	|---------------------------------------------------------------------------------------------------
	*/
    public function cookies_accepted(){
        $cookie_value = Cookie::get( $this->get_cookie_name(), true );
        if( is_array( $cookie_value ) ){
            if( isset( $cookie_value['decline'] ) ){
                return false;
            } elseif( isset( $cookie_value['groups'] ) ){
                return true;
            }
        }
        return false;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Comprueba si la cookie existe en el navegador y ha rechazado
	|---------------------------------------------------------------------------------------------------
	*/
    public function cookies_declined(){
        $cookie_value = Cookie::get( $this->get_cookie_name(), true );
        if( is_array( $cookie_value ) ){
            if( isset( $cookie_value['decline'] ) ){
                return true;
            }
        }
        return false;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Get cookies por servicio
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_services_regex(){
        $services_regex = array(
            'facebook' => '',
            'twitter' => '',
            'youtube' => '#<iframe[^>]*src\s*=\s*\"?https?:\/\/[^\s\"\/]*youtube.com(?:\/[^\s\"]*)?\"?[^>]*>.*?<\/iframe>#i',
            'vimeo' => '#<iframe[^>]*src\s*=\s*\"?https?:\/\/[^\s\"\/]*player.vimeo.com(?:\/[^\s\"]*)?\"?[^>]*>.*?<\/iframe>#i',
            'soundcloud' => '#<iframe[^>]*src\s*=\s*\"?https?:\/\/[^\s\"\/]*w.soundcloud.com\/player(?:\/[^\s\"]*)?\"?[^>]*>.*?<\/iframe>#i',
        );
        return $services_regex;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Get cookies por servicio
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_cookies_by_service( $service ){
        $service_cookies = array(
            'woocommerce' => array( 'woocommerce_cart_hash', 'woocommerce_items_in_cart', 'wp_woocommerce_session_*' ),
            'buddypress' => array( 'bp-*' ),
            'google_analytics' => array( 'NID', 'SID', '_ga', '_gat', '_gat_*', '_gid', '__utma', '__utmb', '__utmc', '__utmt*', '__utmz', '__utmv', '__utmx', '__utmxx' ),
            'google_maps' => array( '1P_JAR', 'NID', 'OGP', 'DV' ),
            'google_tag_manager' => array( 'gtm*', '_dc_gtm*' ),
            'facebook' => array( 'locale', 'datr', 'lsd', 'reg_fb_gate', 'reg_fb_ref', 'reg_ext_ref', 'wb', 'act', '_e_3qqF_0', 'wd', 'c_user', 'csm', 'fr', 'lu', 'xs', 'presence' ),
            'twitter' => array( 'guest_id', 'pid', '__utmv' ),
            'youtube' => array( 'YSC', 'GPS', 'PREF', 'VISITOR_INFO1_LIVE', 'YT-PLAYER-VOLUME', 'use_hitbox', 'recently_watched_video_id_list', 'exchange_uid' ),
            'vimeo' => array( 'vuid', 'player' ),
            'linkedin' => array( 'lidc', 'bcookie', 'lang', 'BizoID', 'BizoData', 'Bizo*' ),
            'hotjar' => array( 'hjClosedSurveyInvites', '_hj*' ),
            'doubleclick' => array( '_dc_gtm_*', 'IDE' ),
            'alexa' => array( '__asc', '__auc' ),
            'cloudflare' => array( '__cfduid' ),
            'sharethis' => array( '__unam' ),
            'addtoany_share_buttons' => array( 'add-to-any' ),
            'clickdimensions' => array( 'cuvid', 'cusid', 'cuvon' ),
            'mailerlite' => array( 'cfduit_*', 'intercom-id', 'intercom-lou', 'mailerlite*' ),
        );
        return isset( $service_cookies[$service] ) ? $service_cookies[$service] : array();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Servicios
	|---------------------------------------------------------------------------------------------------
	*/
    public static function get_services(){
        $services = array(
            'woocommerce' => 'WooCommerce',
            'buddypress' => 'BuddyPress',
            'google_analytics' => 'Google Analytics',
            'google_tag_manager' => 'Google Tag Manager',
            'google_maps' => 'Google Maps',
            'facebook' => 'Facebook',
            'twitter' => 'Twitter',
            'youtube' => 'Youtube',
            'vimeo' => 'Vimeo',
            'soundcloud' => 'SoundCloud',
            'linkedin' => 'Linkedin',
            'hotjar' => 'Hotjar',
            'doubleclick' => 'DoubleClick',
            'alexa' => 'Alexa',
            'cloudflare' => 'Cloudflare',
            'sharethis' => 'ShareThis',
            'addtoany_share_buttons' => 'AddToAny Share Buttons',
            'clickdimensions' => 'ClickDimensions',
            'mailerlite' => 'Mailerlite',
        );
        return Functions::sort( $services );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Cookie Consent Options
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_options( $public = true ){
        $allowed_cookies = $this->get_allowed_cookies();
        if( $public ){
            if( ( $key = array_search( session_name(), $allowed_cookies ) ) !== false ){
                unset( $allowed_cookies[$key] );
            }
        }
        $options = array(
            'cookies_accepted' => $this->cookies_accepted(),
            'cookies_declined' => $this->cookies_declined(),
            'enabled' => $this->is_enabled(),
            'enabled_by_main_option' => Options::option( 'cookie-consent-enabled' ) == 'on',
            'enabled_by_countries' => $this->should_enabled_on_countries(),
            'mode' => Options::option( 'cookie-consent-mode' ),
            'cookie_name' => $this->get_cookie_name(),
            'cookie_duration' => Options::option( 'cookie-consent-cookie-duration' ),
            'close_popup_on_save' => Options::option( 'close-popup-on-save-cookies' ) == 'on' ? true : false,
            'remove_cookies_after_decline' => Options::option( 'remove-cookies-after-decline' ) == 'on' ? true : false,
            'allowed_cookies' => array_values( $allowed_cookies ),
            'all_groups' => array_values( $this->get_enabled_groups() ),
            'groups' => array(),
        );
        $groups = array();
        foreach( $this->get_enabled_groups() as $i => $group_id ){
            $always_active = $i == 1 ? true : false;
            $groups[$group_id] = array(
                'title' => Options::option( 'group-' . $i . '-title' ),
                'always_active' => $always_active,
                'consent_given' => $this->is_consent_given_to_group( $i ),
            );
            if( $i == 1 ){
                $groups[$group_id]['allowed_cookies'] = $this->get_allowed_cookies_by_group( $i );
            } else{
                $groups[$group_id]['blocked_cookies'] = $this->get_blocked_cookies_by_group( $i );
            }
        }
        $options['groups'] = $groups;
        return $options;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna los servicios bloqueados
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_blocked_services(){
        $all_blocked_services = array();
        $all_blocked_services[1] = array();//El grupo 1 no tiene servicios bloqueados
        for( $i = 2; $i <= CPMPP_TOTAL_GROUPS_COOKIE_CONSENT; $i++ ){
            $blocked_services = (array) Options::option( 'group-' . $i . '-blocked-service-cookies', array() );
            $all_blocked_services[$i] = array_unique( array_filter( $blocked_services ) );
        }
        return $all_blocked_services;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna los grupos habilitados
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_enabled_groups(){
        $enabled_groups = array();
        for( $i = 1; $i <= CPMPP_TOTAL_GROUPS_COOKIE_CONSENT; $i++ ){
            if( Options::option( 'group-' . $i . '-enabled' ) == 'on' ){
                $enabled_groups[$i] = 'group_' . $i;
            }
        }
        return $enabled_groups;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si un grupo está habilitado
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_group_enabled( $group_id = 1 ){
        return in_array( $group_id, array_keys( $this->get_enabled_groups() ) );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna los grupos de cookies permitidos
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_allowed_groups(){
        $cookie_value = Cookie::get( $this->get_cookie_name(), true );
        //Desde cookie
        if( $this->cookies_accepted() ){
            $allowed_groups = str_replace( ' ', '', $cookie_value['groups'] );
        } else{//Desde plugin settings
            $allowed_groups = 'group_1';//Grupo 1 siempre se permite
            foreach( $this->get_enabled_groups() as $i => $group_id ){
                //Grupo 1 no tiene esta opción, por lo tanto será false
                if( Options::option( 'group-' . $i . '-initial-consent' ) == 'on' ){
                    $allowed_groups .= ',group_' . $i;
                }
            }
        }
        return array_map( 'trim', explode( ',', $allowed_groups ) );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna las cookies necesarias que nunca serán bloqueadas
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_allowed_cookies(){
        $group_1_cookies = $this->get_allowed_cookies_by_group( 1 );
        $neccesary_cookies = array_merge( $group_1_cookies, $this->system_cookies );
        $neccesary_cookies = apply_filters( 'cookieplus_necessary_cookies', $neccesary_cookies );
        return array_values( array_unique( array_filter( $neccesary_cookies ) ) );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna un array de cookies a permitir de un grupo específico
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_allowed_cookies_by_group( $group_id ){
        if( $group_id != 1 ){
            return array();
        }
        $allowed_cookies = array_map( 'trim', explode( ',', Options::option( 'group-' . $group_id . '-allowed-cookies' ) ) );
        $allowed_services = Options::option( 'group-' . $group_id . '-allowed-service-cookies' );
        if( is_array( $allowed_services ) & ! empty( $allowed_services ) ){
            foreach( $allowed_services as $service ){
                $service_cookies = $this->get_cookies_by_service( $service );
                if( ! empty( $service_cookies ) ){
                    $allowed_cookies = array_merge( $allowed_cookies, $service_cookies );
                }
            }
        }
        $allowed_cookies = apply_filters( 'cookieplus_allowed_cookies_group_' . $group_id, $allowed_cookies );
        return array_values( array_unique( array_filter( $allowed_cookies ) ) );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna las cookies bloqueadas
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_blocked_cookies(){
        $allowed_groups = $this->get_allowed_groups();
        $blocked_groups = array_values( array_diff( array_values( $this->get_enabled_groups() ), $allowed_groups ) );
        if( empty( $blocked_groups ) ){
            return array();
        }
        $blocked_cookies = array();
        foreach( $blocked_groups as $index => $group ){
            if( $group != 'all' && $group != 'group_1' ){
                $group_id = str_replace( 'group_', '', $group );
                $blocked_cookies = array_merge( $blocked_cookies, $this->get_blocked_cookies_by_group( $group_id ) );
            }
        }
        $blocked_cookies = apply_filters( 'cookieplus_blocked_cookies', $blocked_cookies );
        return array_values( array_unique( array_filter( $blocked_cookies ) ) );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna un array de cookies a bloquear de un grupo específico
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_blocked_cookies_by_group( $group_id ){
        if( $group_id == 1 ){
            return array();
        }
        $blocked_cookies = array_map( 'trim', explode( ',', Options::option( 'group-' . $group_id . '-blocked-cookies' ) ) );
        $blocked_services = $this->get_blocked_services()[$group_id];
        if( is_array( $blocked_services ) & ! empty( $blocked_services ) ){
            foreach( $blocked_services as $service ){
                $service_cookies = $this->get_cookies_by_service( $service );
                if( ! empty( $service_cookies ) ){
                    $blocked_cookies = array_merge( $blocked_cookies, $service_cookies );
                }
            }
        }
        $blocked_cookies = apply_filters( 'cookieplus_blocked_cookies_group_' . $group_id, $blocked_cookies );
        return array_values( array_unique( array_filter( $blocked_cookies ) ) );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega scripts en el head
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_scripts_to_head(){
        if( is_admin() || is_feed() || is_robots() || is_trackback() ){
            return;
        }
        $head_scripts = "\n<!-- [BEGIN] MPP:CookiePlus:Header Scripts -->\n";
        $head_scripts .= $this->load_scripts( 'header' );
        $head_scripts .= "\n<!-- [END] MPP:CookiePlus:Header Scripts -->\n";
        echo $head_scripts;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega scripts en el footer
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_scripts_to_footer(){
        if( is_admin() || is_feed() || is_robots() || is_trackback() ){
            return;
        }
        $head_scripts = "\n<!-- [BEGIN] MPP:CookiePlus:Footer Scripts -->\n";
        $head_scripts .= $this->load_scripts( 'footer' );
        $head_scripts .= "\n<!-- [END] MPP:CookiePlus:Footer Scripts -->\n";
        echo $head_scripts;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Agrega scripts en el head
    |---------------------------------------------------------------------------------------------------
    */
    public function load_scripts( $context ){
        $consent_mode = Options::option( 'cookie-consent-mode' );
        $scripts = '';
        if( ! $this->is_enabled() ){
            foreach( $this->get_enabled_groups() as $i => $group_id ){
                $scripts .= $this->get_scripts_by_group( $i, $context );
            }
        } else{
            if( $consent_mode == 'block-all' ){
                //Si ya aceptó las cookies, cargar todos los scripts
                if( $this->cookies_accepted() ){
                    foreach( $this->get_enabled_groups() as $i => $group_id ){
                        $scripts .= $this->get_scripts_by_group( $i, $context );
                    }
                }//Si no, sólo cargar los scripts necesarios (Grupo 1)
                else{
                    $scripts = $this->get_scripts_by_group( 1, $context );
                }
            } else if( $consent_mode == 'by-groups' ){
                foreach( $this->get_enabled_groups() as $i => $group_id ){
                    //Si es por grupos, cargar sólo los grupos aceptados
                    if( $this->is_consent_given_to_group( $i ) ){
                        $scripts .= $this->get_scripts_by_group( $i, $context );
                    }
                }
            }
        }

        return apply_filters( "cookieplus_{$context}_scripts", $scripts );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna scripts de un grupo determinado
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_scripts_by_group( $group_id, $context = 'header' ){
        $group_title = Options::option( 'group-' . $group_id . '-title' );
        $scripts = do_shortcode( Options::option( 'group-' . $group_id . '-' . $context . '-scripts' ) );
        $scripts = apply_filters( "cookieplus_{$context}_scripts_group_{$group_id}", $scripts );

        $return = "\n<!-- >>>>>>>>>> (Gropup $group_id - $group_title) <<<<<<<<<< -->\n";
        $return .= $scripts;
        $return .= "\n<!-- >>>>>>>>>> End (Gropup $group_id - $group_title) <<<<<<<<<< -->\n";

        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Agrega scripts en el head
    |---------------------------------------------------------------------------------------------------
    */
    public function filter_integration_scripts(){
        $integrations = array(
            'facebook', 'google_analytics', 'google_tag_manager', 'google_adwords', 'hotjar', 'custom',
        );
        foreach( $integrations as $integration ){
            $enabled = Options::option( $integration . '-enabled' );
            $group = Options::option( $integration . '-group' );
            $in_footer = Options::option( $integration . '-in-footer' );
            $context = $in_footer ? 'footer' : 'header';
            if( $enabled == 'off' ){
                continue;
            }
            if( $this->is_enabled() ){
                if( $group == 'block-until-accept' && $this->cookies_accepted() ){
                    if( method_exists( $this, "load_{$integration}_code" ) ){
                        add_filter( "cookieplus_{$context}_scripts", array( $this, "load_{$integration}_code" ) );
                    }
                } else if( method_exists( $this, "load_{$integration}_code" ) ){
                    add_filter( "cookieplus_{$context}_scripts_{$group}", array( $this, "load_{$integration}_code" ) );
                }
            } else{
                if( $group == 'block-until-accept' ){
                    if( method_exists( $this, "load_{$integration}_code" ) ){
                        add_filter( "cookieplus_{$context}_scripts", array( $this, "load_{$integration}_code" ) );
                    }
                } else if( method_exists( $this, "load_{$integration}_code" ) ){
                    add_filter( "cookieplus_{$context}_scripts_{$group}", array( $this, "load_{$integration}_code" ) );
                }
            }

        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Scripts de los servicios integrados
	|---------------------------------------------------------------------------------------------------
	*/
    public function load_facebook_code( $scripts ){
        return $scripts . $this->get_integration_code( 'Facebook', 'facebook' );
    }

    public function load_google_tag_manager_code( $scripts ){
        return $scripts . $this->get_integration_code( 'Google Tag manager', 'google_tag_manager' );
    }

    public function load_google_analytics_code( $scripts ){
        return $scripts . $this->get_integration_code( 'Google Analytics', 'google_analytics' );
    }

    public function load_google_adwords_code( $scripts ){
        return $scripts . $this->get_integration_code( 'Google Adwords', 'google_adwords' );
    }

    public function load_hotjar_code( $scripts ){
        return $scripts . $this->get_integration_code( 'Hot Jar', 'hotjar' );
    }

    public function load_custom_code( $scripts ){
        return $scripts . $this->get_integration_code( 'Custom Integration', 'custom' );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna código script de los servicios integrados
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_integration_code( $integration_title, $integration_name ){
        $script_title = "$integration_title loaded by {$this->plugin->arg('menu_title')}";
        $script = "\n<!-- $script_title -->\n";
        $script .= do_shortcode( Options::option( $integration_name . '-code' ) );
        $script .= "\n<!-- End $script_title -->\n";
        return $script;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Comprueba si se debe desactivar google analytics
	|---------------------------------------------------------------------------------------------------
	*/
    public function maybe_disable_google_analytics(){
        return $this->is_enabled() && ! $this->cookies_accepted() && Options::option( 'google_analytics-enabled' ) == 'on' && Options::option( 'google_analytics-id' );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Procesa las cookies
	|---------------------------------------------------------------------------------------------------
	*/
    public function process_header_cookies(){
        if( Functions::is_login_page() || ! $this->is_enabled() ){
            return;
        }

        //Cookie::set_encoded( 'TestCookieName', 'TestCookieValue', 1, '/' );

        $headers_list = headers_list();
        $header_cookie_names = array();
        $header_cookies = array();
        foreach( $headers_list as $header ){
            if( strpos( $header, 'Set-Cookie' ) !== false ){
                $header_cookies[] = $header;
                $pairs = explode( '=', $header );
                $cookie_name = trim( str_replace( array( 'Set-Cookie', ':' ), '', $pairs[0] ) );
                $header_cookie_names[] = $cookie_name;
            }
        }

        if( empty( $header_cookie_names ) ){
            return;
        }

        $cookies_to_set = array();
        $plugin_cookies = $this->plugin->get_plugin_cookies( false );
        $allowed_cookies = $this->get_allowed_cookies();
        $consent_mode = Options::option( 'cookie-consent-mode' );
        $cookie_value = Cookie::get( $this->get_cookie_name(), true );

        foreach( $header_cookie_names as $key => $cookie_name ){
            //Si la cookie es una cookie del plugin, permitir crearla.
            if( Functions::in_array( $cookie_name, $plugin_cookies, true ) ){
                $cookies_to_set[] = $header_cookies[$key];
                continue;
            }
            if( $consent_mode == 'block-all' ){
                //Si ya aceptó las cookies, permitir crear cualquier cookie
                if( $this->cookies_accepted() ){
                    $cookies_to_set[] = $header_cookies[$key];
                }//Si no, sólo permitir las cookies necesarias
                else if( Functions::in_array( $cookie_name, $allowed_cookies, true ) ){
                    $cookies_to_set[] = $header_cookies[$key];
                }
            } else if( $consent_mode == 'by-groups' ){
                //Si ya aceptó las cookies, y aceptó todos los grupos, permitir todas las cookies
                if( $this->cookies_accepted() && $cookie_value['groups'] == 'all' ){
                    $cookies_to_set[] = $header_cookies[$key];
                } //Si aún no acepta cookies o no aceptó todos los grupos, bloquear las cookies de los grupos no aceptados
                else if( ! Functions::in_array( $cookie_name, $this->get_blocked_cookies(), true ) ){
                    $cookies_to_set[] = $header_cookies[$key];
                }
            }
        }

        //$this->d("Test", "");
        $this->add_allowed_cookies_to_header( $cookies_to_set, true );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega las cookies permitidas al header
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_allowed_cookies_to_header( $allowed_cookies, $remove_all = true ){
        if( headers_sent() ){
            return;
        }
        if( $remove_all ){
            header_remove( 'Set-Cookie' );
        }
        foreach( $allowed_cookies as $cookie ){
            header( $cookie, false );
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Procesa el consentimiento de cookies
	|---------------------------------------------------------------------------------------------------
	*/
    public function proccess_cookie_consent(){
        $response = array(
            'success' => true,
        );

        if( ! isset( $_POST['cookie_consent'] ) || ! isset( $_POST['accept_all'] ) || ! isset( $_POST['decline'] ) ){
            $response['success'] = false;
            wp_send_json( $response );
        }


        $decline = $_POST['decline'];

        if( $decline == 'true' || $decline === true ){
            $response = $this->decline_cookies();
        } else{
            $response = $this->accept_cookies();
        }

        //Cache plugin
        if( function_exists( 'w3tc_flush_all' ) ){
            w3tc_flush_all();
        }

        //Consent Logs
        $consent_log = $this->plugin->register_consent_log( $_POST, $response['cookie_value'] );
        $response = array_merge( $response, $consent_log);

        wp_send_json( $response );

    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Aceptar cookies
	|---------------------------------------------------------------------------------------------------
	*/
    public function accept_cookies(){
        $response = array(
            'success' => true,
            'action' => 'accept',
        );

        $accept_all = $_POST['accept_all'];
        $groups = $_POST['cookie_consent']['groups'];
        $cookie_name = $this->get_cookie_name();
        $expiration_time = $this->expiration_time();

        $cookie_value = array(
            'groups' => '',
            'expiration_time' => $expiration_time,
            'date_created' => current_time( 'mysql' ),
        );

        if( $accept_all == 'true' || $accept_all === true ){
            $cookie_value['groups'] = 'all';
        } else{
            $total_accepted = 0;
            foreach( $groups as $key => $group ){
                if( $group['consent_given'] == 'true' || $group['consent_given'] === true ){
                    $cookie_value['groups'] .= $key . ',';
                    $total_accepted++;
                }
            }
            $cookie_value['groups'] = rtrim( $cookie_value['groups'], ',' );
        }
        //Las cookies se agregan desde javascript para evitar httponly=true
        //$cookie_value_encoded = Cookie::set_encoded( $cookie_name, $cookie_value, $expiration_time, '/' );

        if( is_user_logged_in() ){
            update_user_meta( get_current_user_id(), $cookie_name, $cookie_value );
        }

        $response['cookie_value'] = $cookie_value;
        //$response['cookie_value_encoded'] = $cookie_value_encoded;
        return $response;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Tiempo de expiración de la cookie
	|---------------------------------------------------------------------------------------------------
	*/
    public function expiration_time(){
        $cookie_duration = Options::option( 'cookie-consent-cookie-duration' );
        $expiration_time = time() + Functions::days_to_seconds( $cookie_duration );
        return $expiration_time;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Rechazar cookies
	|---------------------------------------------------------------------------------------------------
	*/
    public function decline_cookies(){
        $response = array(
            'success' => true,
            'action' => 'decline',
        );
        $cookie_name = $this->get_cookie_name();
        $expiration_time = $this->expiration_time();
        $cookie_value = array(
            'groups' => 'group_1',
            'expiration_time' => $expiration_time,
            'date_created' => current_time( 'mysql', 0 ),
            'decline' => '1',
        );

        //Las cookies se agregan desde javascript para evitar httponly=true
        //$cookie_value_encoded = Cookie::set_encoded( $cookie_name, $cookie_value, $expiration_time, '/' );

        if( is_user_logged_in() ){
            update_user_meta( get_current_user_id(), $cookie_name, $cookie_value );
        }

        $response['cookie_value'] = $cookie_value;
        //$response['cookie_value_encoded'] = $cookie_value_encoded;
        return $response;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Shortcode para bloquear contenido
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_block_content_shortcode( $atts = '', $content = null ){
        $atts = shortcode_atts( array(
            'group' => '-1',
            'message' => Options::option( 'shortcode-block-content-message' ),
        ), $atts );

        $allow_content = $this->allow_content_for_group( $atts['group'] );

        if( $allow_content ){
            $content = do_shortcode( $content );
        } else{
            $text = Options::option( 'shortcode-accept-cookies-text' );
            $button = do_shortcode( '[cookieplus_btn_accept_cookies]' . $text . '[/cookieplus_btn_accept_cookies]' );
            $message = "<div class='cookieplus-blocked-cookies-message'>{$atts['message']}</div>";
            $content = "<div class='cookieplus'>{$message}{$button}</div>";
        }

        return $content;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Botón para aceptar cookies
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_button_accept_cookies_shortcode( $atts = '', $content = null ){
        if( ! $this->is_enabled() ){
            return '';
        }

        $atts = shortcode_atts( array(
            'id' => '',
            'class' => '',
            'radius' => '5px',
            'style' => '',
            'hide_if_accepted' => 'false',
        ), $atts );

        if( $atts['hide_if_accepted'] == 'true' && $this->cookies_accepted() ){
            return '';
        }

        $button = '';
        $id = ! empty( $atts['id'] ) ? "id='{$atts['id']}'" : '';
        $class = 'cookieplus-btn cookieplus-btn-green ' . $atts['class'];
        $button .= "<div $id class='$class cookieplus-btn-accept-cookies' style='border-radius: {$atts['radius']}; {$atts['style']}'>";
        $button .= do_shortcode( $content );
        $button .= "</div>";

        return $button;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Botón para rechazar cookies
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_button_decline_cookies_shortcode( $atts = '', $content = null ){
        if( ! $this->is_enabled() ){
            return '';
        }

        $atts = shortcode_atts( array(
            'id' => '',
            'class' => '',
            'radius' => '5px',
            'style' => '',
            'hide_if_not_accepted' => 'false'
        ), $atts );

        if( $atts['hide_if_not_accepted'] == 'true' && ! $this->cookies_accepted() ){
            return '';
        }

        $button = '';
        $id = ! empty( $atts['id'] ) ? "id='{$atts['id']}'" : '';
        $class = 'cookieplus-btn cookieplus-btn-red ' . $atts['class'];
        $button .= "<div $id class='cookieplus-btn-decline-cookies $class' style='border-radius: {$atts['radius']}; {$atts['style']}'>";
        $button .= do_shortcode( $content );
        $button .= "</div>";

        return $button;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Comprueba si ya se dio consentimiento a la cookie
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_consent_given_to_group( $group_id ){
        if( $group_id == 1 ){
            return true;
        }
        $initial_consent = Options::option( 'group-' . $group_id . '-initial-consent' );
        $cookie_value = Cookie::get( $this->get_cookie_name(), true );

        if( ! $this->cookies_accepted_or_declined() ){
            return $initial_consent == 'on' ? true : false;
        }
        $groups = $cookie_value['groups'];
        if( $groups == 'all' ){
            return true;
        } else{
            return strpos( $groups, 'group_' . $group_id ) !== false;
        }
        return false;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si se debe permitir contenido para un grupo en específico
	|---------------------------------------------------------------------------------------------------
	*/
    public function allow_content_for_group( $group_id ){
        $allow_content = false;
        $consent_mode = Options::option( 'cookie-consent-mode' );

        if( ! $this->is_enabled() ){
            $allow_content = true;
        } else{
            if( $consent_mode == 'block-all' ){
                //Si es modo 'block-all', sólo verificar cookie
                $allow_content = $this->cookies_accepted();
            } else if( $consent_mode == 'by-groups' ){
                if( $group_id == '-1' ){
                    $allow_content = $this->cookies_accepted();
                } else{
                    $allow_content = $this->is_consent_given_to_group( $group_id );
                }
            }
        }
        return $allow_content;
    }


}