<?php namespace MasterPopupsAddonCookiePlus\Includes;

use MasterPopupsAddonCookiePlus\Includes\Integrations as Integrations;
use MasterPopups\Includes\Functions as MasterPopupsFunctions;
use MasterPopups\Includes\Assets;

class CookiePlus {
    protected static $instance = null;
    public static $args = array();
    public $Options = null;
    public $geo = null;
    public $SettingsPanel = null;
    public $TabCookieSettings = null;
    public $TermsConditions = null;
    public $PrivacyPolicy = null;
    public $CookieConsent = null;
    public $post_types = array();

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $version = '1.0.0' ){
        if( null === self::$instance ){
            self::$instance = new self( $version );

        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    private function __construct( $args = array() ){
        self::$args = $args;

        $this->post_types = $this->arg( 'post_types' );

        $this->constants();
        $this->plugin_loader();
        $this->hooks();

    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Debug
	|---------------------------------------------------------------------------------------------------
	*/
    public function d( $var1, $var2 = null ){
        echo '<pre style="margin-left: 200px; margin-top: 20px; background: white;">';
        print_r( $var1 );
        echo '</pre>';
        if( $var2 !== null ){
            echo '<pre style="margin-left: 200px; margin-top: 20px; background: white;">';
            print_r( $var2 );
            echo '</pre>';
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin arguments
	|---------------------------------------------------------------------------------------------------
	*/
    public function arg( $name = '', $key = '' ){
        if( isset( self::$args[$name] ) ){
            if( $key ){
                if( isset( self::$args[$name][$key] ) ){
                    return self::$args[$name][$key];
                } else{
                    return null;
                }
            }
            return self::$args[$name];
        }
        return null;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constants
	|---------------------------------------------------------------------------------------------------
	*/
    public function constants(){
        define( 'CPMPP_VERSION', $this->arg( 'version' ) );
        define( 'CPMPP_DIR', trailingslashit( dirname( dirname( __FILE__ ) ) ) );
        define( 'CPMPP_URL', trailingslashit( plugins_url( '', dirname( __FILE__ ) ) ) );
        define( 'CPMPP_TOTAL_GROUPS_COOKIE_CONSENT', 5 );


        //Coocie Consent
        define( 'CPMPP_KEY_COOKIE_CONSENT', 'cookie_consent' );
        define( 'CPMPP_KEY_TERMS_CONSENT', 'accept_terms' );
        define( 'CPMPP_KEY_PRIVACY_CONSENT', 'accept_policy' );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si Master Popups está activo
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_masterpopups_active(){
        $loader = \MasterPopupsAddonCookiePlus\Loader::get_instance();
        $plugin_data = $loader->get_plugin_data();
        return ! empty( $plugin_data );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin loader
	|---------------------------------------------------------------------------------------------------
	*/
    private function plugin_loader(){
        include dirname( __FILE__ ) . '/class-autoloader.php';
        include CPMPP_DIR . 'libs/GeoIP2-php/vendor/autoload.php';
        ClassAutoloader::run();

        if( ! $this->is_masterpopups_active() ){
            return;
        }

        $this->geo = new GeoDetection( CPMPP_DIR . 'libs/GeoLite2-Country/GeoLite2-Country.mmdb' );

        AssetsLoader::get_instance( $this );
        Ajax::get_instance( $this );
        $this->SettingsPanel = SettingsPanel::get_instance( $this );
        $this->Options = Options::get_instance( $this );
        $this->CookieConsent = CookieConsent::get_instance( $this );
        $this->TabCookieSettings = TabCookieSettings::get_instance( $this );
        $this->TermsConditions = TermsConditions::get_instance( $this );
        $this->PrivacyPolicy = PrivacyPolicy::get_instance( $this );

        Integrations\WpComments::get_instance( $this );
        Integrations\Woocommerce::get_instance( $this );
        Integrations\Buddypress::get_instance( $this );
        Integrations\CF7::get_instance( $this );
        Integrations\GravityForms::get_instance( $this );

    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_action( 'init', array( $this, 'create_post_types' ) );
        add_action( 'plugins_loaded', array( $this, 'plugin_textdomain' ) );
        add_action( 'admin_notices', array( $this, 'show_message_to_install_masterpopups' ) );
        add_filter( "mpp_add_popup_templates", array( $this, 'add_popup_templates' ), 10, 2 );

        add_action( 'wp_head', array( $this, 'load_resources_in_header' ), 10 );
        add_action( 'admin_head', array( $this, 'load_resources_in_header' ), 10 );


        //add_action( 'admin_head', array( $this, 'load_resources_in_header' ), 10 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin text domain
	|---------------------------------------------------------------------------------------------------
	*/
    public function plugin_textdomain(){
        $plugin_rel_path = trailingslashit( plugin_basename( CPMPP_DIR ) );
        $loaded = load_plugin_textdomain( 'mpp-cookieplus', false, $plugin_rel_path . 'languages/' );

        if( ! $loaded ){
            load_textdomain( 'mpp-cookieplus', CPMPP_DIR . 'languages/cookieplus-' . get_locale() . '.mo' );
        }
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Cookie Plus Popup Templates
	|---------------------------------------------------------------------------------------------------
	*/
    public function show_message_to_install_masterpopups(){
        $plugin_name = $this->arg( 'name' );
        if( ! $this->is_masterpopups_active() ){
            $message = sprintf( 'You have to install and activate %sMaster Popups%s plugin before using %s"' . $plugin_name . '"%s', '<a href="http://bit.ly/2HQjoKF" target="_blank"><strong>', '</strong></a>', '<strong>', '</strong>' );
            $print = "<h3 style='margin: 10px 0px; font-size:1.1em;'>Missing plugin: Master Popups</h3>";
            $print .= "<p>$message</p>";
            echo "<div class='notice notice-warning update-nag' style='display:block;'>$print</div>";
        } else if( version_compare( MPP_VERSION, '2.0.0', '<' ) ){
            $message = sprintf( 'You have an old version of %sMaster Popups%s you need version 2.0.0 or higher to use %s"' . $plugin_name . '"%s', '<a href="http://bit.ly/2HQjoKF" target="_blank"><strong>', '</strong></a>', '<strong>', '</strong>' );
            $print = "<h3 style='margin: 10px 0px; font-size:1.1em;'>{$this->arg('menu_title')} Message:</h3>";
            $print .= "<p>$message</p>";
            echo "<div class='notice notice-warning update-nag' style='display:block;'>$print</div>";
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna la IP del usuario actual o una IP falsa agregada desde el debug de MasterPopups
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_real_or_fake_ip(){
        $ip = null;
        $debug_mode = Options::option( 'debug-mode' );
        $debug_ip = Options::option( 'debug-ip' );
        if( $debug_mode == 'on' && $this->geo && $this->geo->is_valid_ip( $debug_ip ) ){
            $ip = $debug_ip;
        }
        if( ! $ip && $this->geo ){
            $ip = $this->geo->get_current_ip();
        }
        return $ip;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Geo info
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_geo_info( $ip = null ){
        if( ! $this->geo ){
            return null;
        }
        if( ! $ip ){
            $ip = $this->geo->get_current_ip();
        }
        //$ip = '47.73.0.129';//Germany
        //$ip = '85.205.244.0';//Italy
        //$ip = '122.152.142.128';//Japan
        //$ip = '143.255.143.180';//Paraguay
        //$ip = '138.121.232.0';//Brazil
        //$ip = '104.132.117.0';//France
        //$ip = '141.243.0.0';//Australia
        //$ip = '128.101.101.101';//United States
        //$ip = '213.192.202.121';//Spain
        //$ip = '190.233.77.225';//Peru
        return $this->geo->get_info( $ip );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna la IP enmascarada si así se estableció en los ajustes
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_masked_ip( $ip = '', $store = true ){
        $ip_masking = Options::option( 'logs-ip-masking' );
        $real_or_fake_ip = $ip ? $ip : $this->get_real_or_fake_ip();
        $return_ip = $real_or_fake_ip;

        if( $store ){
            if( $ip_masking == 'store-full-show-full' || $ip_masking == 'store-full-show-masked' ){
                $return_ip = $real_or_fake_ip;
            } else if( $ip_masking == 'store-masked-show-masked' ){
                if( $this->geo->is_valid_ip( $real_or_fake_ip ) ){
                    $return_ip = $this->geo->mask_ip( $real_or_fake_ip, 'X' );
                } else{
                    $return_ip = $real_or_fake_ip;
                }
            }
        } else{
            if( $ip_masking == 'store-full-show-full' ){
                $return_ip = $real_or_fake_ip;
            } else if( $ip_masking == 'store-full-show-masked' || $ip_masking == 'store-masked-show-masked' ){
                //Si ya está enmascarado, solo mostrarlo
                if( stripos( $real_or_fake_ip, 'X' ) !== false ){
                    $return_ip = $real_or_fake_ip;
                } else{
                    $return_ip = $this->geo->mask_ip( $real_or_fake_ip, 'X' );
                }
            }
        }
        return $return_ip;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna todos los posts de tipo Consent Logs
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_consent_logs( $args = array() ){
        $args = wp_parse_args( $args, array(
            'post_type' => $this->post_types['consent_logs'],
            'numberposts' => -1,//=posts_per_page
        ) );
        $posts = get_posts( $args );
        return $posts;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Registra los consentimientos de cookies, terminos & condiciones y políticas de privacidad
	|---------------------------------------------------------------------------------------------------
	*/
    public function register_consent_log( $post_data = array(), $cookie_value = array() ){
        $prefix = $this->arg( 'prefix' );
        $type = str_replace( 'cookieplus_', '', $post_data['action'] );
        $title = 'Consent Log: ' . $type;
        $admin_ids = Functions::admin_user_ids( true );
        $admin_id = empty( $admin_ids ) ? get_current_user_id() : $admin_ids[0];
        $content = do_shortcode( Options::option( 'cookie-settings-content' ) );
        $content = wpautop( $content );

        $post_log = array(
            'post_type' => $this->post_types['consent_logs'],
            'post_title' => wp_strip_all_tags( $title ),
            'post_content' => $content,
            'post_status' => 'publish',
            'post_author' => $admin_id,
        );

        $post_id = wp_insert_post( $post_log );

        $update_fields = array(
            'type' => $type,
        );

        //Accept info
        $update_fields['declined'] = isset( $cookie_value['decline'] ) ? true : false;
        $update_fields['version'] = isset( $cookie_value['version'] ) ? $cookie_value['version'] : '1.0';
        $cookie_names = $this->get_plugin_cookies();

        $cv = $cookie_value;
        if( $type == CPMPP_KEY_COOKIE_CONSENT ){
            $update_fields['accepted'] = ! $update_fields['declined'];
            $update_fields['cookie_name'] = $cookie_names['cookie_consent'];
            $update_fields['expiration_time'] = isset( $cv['expiration_time'] ) ? $cv['expiration_time'] : $this->CookieConsent->expiration_time();
        } else if( $type == CPMPP_KEY_TERMS_CONSENT ){
            $update_fields['accepted'] = true;
            $update_fields['cookie_name'] = $cookie_names['terms_conditions'];
            $update_fields['expiration_time'] = isset( $cv['expiration_time'] ) ? $cv['expiration_time'] : $this->TermsConditions->expiration_time();
        } else{
            $update_fields['accepted'] = true;
            $update_fields['cookie_name'] = $cookie_names['privacy_policy'];
            $update_fields['expiration_time'] = isset( $cv['expiration_time'] ) ? $cv['expiration_time'] : $this->PrivacyPolicy->expiration_time();
        }

        $update_fields['date_created'] = isset( $cookie_value['date_created'] ) ? $cookie_value['date_created'] : current_time( 'mysql', 0 );
        $update_fields['groups'] = isset( $cookie_value['groups'] ) ? $cookie_value['groups'] : null;

        $groups_info = isset( $post_data['cookie_consent']['groups'] ) ? $post_data['cookie_consent']['groups'] : array();
        foreach( $groups_info as $key => &$group ){
            unset( $group['always_active'] );
        }
        $update_fields['groups_info'] = $groups_info;

        //User info
        $update_fields['user_id'] = is_user_logged_in() ? get_current_user_id() : 0;

        $update_fields['user_ip'] = $this->get_masked_ip( $this->get_real_or_fake_ip(), true );
        $update_fields['user_agent'] = isset( $_SERVER['HTTP_USER_AGENT'] ) ? $_SERVER['HTTP_USER_AGENT'] : '';
        $update_fields['page_url'] = Functions::current_url( true );

        //Update fields
        foreach( $update_fields as $key => $value ){
            update_post_meta( $post_id, $prefix . $key, $value );
        }

        return array(
            'post_log' => $post_log,
            'post_data' => $post_data,
            'cookie_value' => $cookie_value,
            //'update_fields' => $update_fields,
        );
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Create custom post types
    |---------------------------------------------------------------------------------------------------
    */
    public function create_post_types(){
        //$this->test_register_consent_log();
        $this->create_post_type_consent_logs();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Test
	|---------------------------------------------------------------------------------------------------
	*/
    public function test_register_consent_log(){
        $types = array(
//            CPMPP_KEY_COOKIE_CONSENT,
//            CPMPP_KEY_TERMS_CONSENT,
//            CPMPP_KEY_PRIVACY_CONSENT
        );
        foreach( $types as $type ){
            $total = 500;
            $i = 0;
            for( $i = 0; $i < $total; $i++){
                $post_data = array(
                    'action' => 'cookieplus_'.$type,
                    'cookie_consent' => array(
                        'groups' => array(
                            'group_1' => array(
                                'allowed_cookies' => array(),
                                'always_active' => true,
                                'consent_given' => true,
                                'title' => 'Necessary Cookies',
                            ),
                            'group_2' => array(
                                'allowed_cookies' => array(),
                                'always_active' => false,
                                'consent_given' => true,
                                'title' => 'Preferences Cookies',
                            ),
                            'group_4' => array(
                                'allowed_cookies' => array(),
                                'always_active' => false,
                                'consent_given' => false,
                                'title' => 'Marketing Cookies',
                            )
                        )
                    )
                );
                $cookie_value = array(
                    'groups' => 'group_1, group_2',
                    'expiration_time' => time() + Functions::days_to_seconds(10),
                    'date_created' => current_time( 'mysql', 0 ),
                    //'decline' => '1',
                    'page_id' => '0',
                    'version' => '',
                );
                $this->register_consent_log( $post_data, $cookie_value );
            }
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Post Type: Consent Logs
    |---------------------------------------------------------------------------------------------------
    */
    public function create_post_type_consent_logs(){
        if( post_type_exists( $this->post_types['consent_logs'] ) ){
            return;
        }

        //Popups
        $singular = __( 'Consent Logs', 'mpp-cookieplus' );
        $plural = __( 'Consent Log', 'mpp-cookieplus' );
        $all_items = sprintf( __( 'All %s', 'mpp-cookieplus' ), $plural );
        $labels = $this->get_post_type_labels( 'Cookie Plus Logs', $plural, $singular, $all_items );
        $args = $this->get_post_type_args( array(
            'labels' => $labels,
            'public' => false,
            'supports' => array( 'title', 'editor' ),
            'show_ui' => false,//Si se debe generar y permitir una interfaz de usuario de administración
            'show_in_menu' => false,//Si se debe mostrar en el menú de administración o como submenú (agregar parent)
            'menu_position' => 20,
            //'menu_icon' => MPP_URL . 'assets/admin/images/icon-plugin2.png',
        ) );
        register_post_type( $this->post_types['consent_logs'], $args );
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Labels for custom post types
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_post_type_labels( $menu_name, $plural, $singular, $all_items ){
        return array(
            'singular_name' => $singular,
            'name' => $plural,
            'menu_name' => $menu_name,
            'add_new' => sprintf( __( 'New %s', 'mpp-cookieplus' ), $singular ),
            'name_admin_bar' => sprintf( '%s', $singular ) . ' (' . $menu_name . ')',
            'all_items' => $all_items,
            'add_new_item' => sprintf( __( 'Add %s', 'mpp-cookieplus' ), $singular ),
            'new_item' => sprintf( __( 'New %s', 'mpp-cookieplus' ), $singular ),
            'edit_item' => sprintf( __( 'Edit %s', 'mpp-cookieplus' ), $singular ),
            'update_item' => sprintf( __( 'Update %s', 'mpp-cookieplus' ), $singular ),
            'view_item' => sprintf( __( 'View %s', 'mpp-cookieplus' ), $singular ),
            'view_items' => sprintf( __( 'View %s', 'mpp-cookieplus' ), $plural ),
            'search_items' => sprintf( __( 'Search %s', 'mpp-cookieplus' ), $plural ),
            'not_found' => sprintf( __( 'No %s found', 'mpp-cookieplus' ), $plural ),
            'not_found_in_trash' => sprintf( __( 'No %s found in Trash', 'mpp-cookieplus' ), $plural ),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Labels for custom post types
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_post_type_args( $args ){
        return array(
            'labels' => $args['labels'],
            'description' => '',
            'supports' => $args['supports'],
            'hierarchical' => false,
            'capability_type' => 'post',

            'public' => $args['public'],//dejar como falso
            'publicly_queryable' => $args['public'],//Permite que sea visible en el front-end. url.com/post-type/popup-slug
            'exclude_from_search' => ! $args['public'],//true
            'show_ui' => $args['show_ui'],
            'show_in_menu' => isset( $args['show_in_menu'] ) ? $args['show_in_menu'] : $args['show_ui'],
            'show_in_nav_menus' => $args['public'],//false. Permite seleccionar desde menús de navegación

            'menu_position' => isset( $args['menu_position'] ) ? $args['menu_position'] : null,
            'menu_icon' => isset( $args['menu_icon'] ) ? $args['menu_icon'] : null,
            'show_in_admin_bar' => isset( $args['show_in_menu'] ) ? $args['show_in_menu'] : $args['show_ui'],
            'can_export' => true,//Si permitir que este tipo de publicación se exporte
            'has_archive' => false,
            'rewrite' => false,//para ocultar permalink al editar
            'delete_with_user' => false
        );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Carga de recursos en header
	|---------------------------------------------------------------------------------------------------
	*/
    public function load_resources_in_header(){
        if( ! $this->is_masterpopups_active() || ! $this->TabCookieSettings ){
            return;
        }

        $return = "";
        $version = $this->arg( 'version' );
        $return .= "\n\n<!-- MPP:CookiePlus v$version -->";
        $return .= "\n\n<!-- [BEGIN] MPP:CookiePlus:Header Styles -->";
        $return .= "\n<style>";
        $return .= "\n/* Custom CSS - Tab Cookie Settings */\n";
        $return .= $this->TabCookieSettings->build_style();
        $return .= "\n/* Custom CSS - General Settings */\n";
        $return .= Options::option( 'custom-css' );
        $return .= "\n</style>";

        //Google font
        if( Options::option( 'use-font-family' ) == 'on' && $font_family = Options::option( 'font-family' ) ){
            $google_fonts = array(
                $font_family => array( '300', '400', '500', '600', '700', '800', '900' ),
            );
            $href = MasterPopupsFunctions::url_google_fonts( $google_fonts, array_values( Assets::local_fonts() ) );
            $link_rel = '<link class="mpp-cookieplus-google-fonts" href="' . $href . '" rel="stylesheet" type="text/css">';
            $return .= "\n$link_rel";
        }

        $return .= "\n<!-- [END] MPP:CookiePlus:Header Styles -->\n\n";
        echo $return;
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna las cookies usadas en el plugin
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_plugin_cookies( $keys = true ){
        $cookie_names = array(
            'cookie_consent' => $this->CookieConsent->get_cookie_name(),
            'terms_conditions' => $this->TermsConditions->get_cookie_name(),
            'privacy_policy' => $this->PrivacyPolicy->get_cookie_name(),
        );
        if( $keys ){
            return $cookie_names;
        }
        return array_values( $cookie_names );
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Cookie Plus Popup Templates
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_popup_templates( $original_templates = array(), $version = '1.0.0' ){
        $templates_url = CPMPP_URL . 'assets/templates/';
        return array(
            array(
                'template' => $templates_url . 'popup-template-001.json',
                'image' => $templates_url . 'popup-template-001.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-002.json',
                'image' => $templates_url . 'popup-template-002.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-003.json',
                'image' => $templates_url . 'popup-template-003.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-004.json',
                'image' => $templates_url . 'popup-template-004.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-005.json',
                'image' => $templates_url . 'popup-template-005.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-006.json',
                'image' => $templates_url . 'popup-template-006.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
        );

    }
}