<?php namespace MasterPopupsAddonCookiePlus\Includes;

use GeoIp2\Database\Reader;

class GeoDetection extends RemoteAddress {
    public $error = '';
    private $database = '';

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    public function __construct( $mmdb_file = '' ){
        if( ! empty( $mmdb_file ) ){
            $this->database = $mmdb_file;
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega la base datos de países
	|---------------------------------------------------------------------------------------------------
	*/
    public function set_database( $mmdb_file ){
        $this->database = $mmdb_file;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna la IP actual
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_current_ip(){
        return $this->getIpAddress();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna la información de la IP actual
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_info_from_current_ip(){
        return $this->get_info( $this->get_current_ip() );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna la información de una IP determinada
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_info( $ip ){
        if( $this->is_valid_ip( $ip ) ){
            try{
                $reader = new Reader( $this->database );
                $country = $reader->country( $ip );
                return $country;
            } catch( \Exception $e ){
                $this->error = $e->getMessage();
            }
        }
        return null;
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si una IP es válida
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_valid_ip( $ip ){
        return filter_var( $ip, FILTER_VALIDATE_IP );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si una IP versión 4 es válida
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_valid_ip_v4( $ip ){
        return filter_var( $ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si una IP versión 6 es válida
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_valid_ip_v6( $ip ){
        return filter_var( $ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Enmascara una IP v4 o v6 con un símbolo
	|---------------------------------------------------------------------------------------------------
	*/
    public function mask_ip( $ip, $s = 'X' ){
        return preg_replace(
            [ '/\.\d*$/', '/::\d*$/', '/[\da-fA-F]+:[\da-fA-F]*$/' ],
            [ ".{$s}{$s}{$s}", "::{$s}{$s}{$s}", "{$s}{$s}{$s}{$s}:{$s}{$s}{$s}{$s}" ],
            $ip );
    }
}