<?php namespace MasterPopupsAddonCookiePlus\Includes;


class Options {
    protected static $instance = null;
    public $plugin;
    public static $defaults = array();
    public static $options = array();
    public static $prefix = '';


    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    private function __construct( $plugin ){
        $this->plugin = $plugin;

        add_action( 'plugins_loaded', array( $this, 'load_options' ), 11 );
        add_action( 'init', array( $this, 'init' ), 2 );//2 ya que en class-settings-panel.php se inició en 1
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Init action
    |---------------------------------------------------------------------------------------------------
    */
    public function init(){
        self::$defaults = SettingsPanel::$defaults;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Carga las opciones. Después del filtro por WPML.('plugins_loaded', 'icl_st_set_admin_options_filters' ), 10 ))
    |---------------------------------------------------------------------------------------------------
    */
    public function load_options(){
        self::$options = get_option( $this->plugin->arg( 'settings_id' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Acceso a cualquier opción
    |---------------------------------------------------------------------------------------------------
    */
    public static function option( $option_name = '', $default_value = null ){
        $value = null;
        $option_name = self::get_option_name( $option_name );
        if( isset( self::$options[$option_name] ) ){
            $value = self::$options[$option_name];
        } else if( isset( self::$defaults[$option_name] ) ){
            $value = self::$defaults[$option_name];
        } else if( $default_value != null ){
            $value = $default_value;
        }
        return $value;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Obtiene el nombre real de la opción
	|---------------------------------------------------------------------------------------------------
	*/
    public static function get_option_name( $name ){
        if( ! Functions::starts_with( self::$prefix, $name ) ){
            return self::$prefix . $name;
        }
        return $name;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Retorna las variables javascript con las opciones del popup
    |---------------------------------------------------------------------------------------------------
    */
    public function get_js_options(){
        $return = '';
        $options = $this->get_plugin_options();
        $return .= "\n\n var CookiePlus = " . json_encode( $options ) . ";";

        return $return;
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Options
	|---------------------------------------------------------------------------------------------------
	*/
    public static function script_options_for_all_cookie_groups( $xbox, $group_id ){
        if( $group_id == 1 ){
            $title_scripts_desc = __( 'These scripts will always be loaded.', 'mpp-cookieplus' );
            $xbox->add_field( array(
                'type' => 'title',
                'name' => __( 'Cookies', 'mpp-cookieplus' ),
                'desc' => __( 'These cookies will always be allowed. Internally, wordpress and login cookies are included.', 'mpp-cookieplus' ),
            ) );
            $xbox->add_field( array(
                'id' => 'group-1-allowed-cookies',
                'name' => __( 'Allowed cookie names', 'mpp-cookieplus' ),
                'desc' => __( 'Enter the cookie names you want to allow (comma separated)', 'mpp-cookieplus' ),
                'type' => 'textarea',
                'default' => self::$defaults['group-1-allowed-cookies'],
                'attributes' => array(
                    'style' => 'min-height: 100px; height: 100px; '
                )
            ) );
            $xbox->add_field( array(
                'id' => 'group-' . $group_id . '-allowed-service-cookies',
                'name' => __( 'Allowed services cookies', 'mpp-cookieplus' ),
                'desc' => __( 'Select the services you want to allow cookies.', 'mpp-cookieplus' ),
                'type' => 'checkbox',
                'default' => self::$defaults['group-' . $group_id . '-allowed-service-cookies'],
                'items' => CookieConsent::get_services(),
            ) );
        } else{
            $title_scripts_desc = __( 'These scripts will be inserted as long as the user accepts this cookie group.', 'mpp-cookieplus' );
            $xbox->add_field( array(
                'type' => 'title',
                'name' => __( 'Cookies', 'mpp-cookieplus' ),
                'desc' => __( 'These cookies will be blocked until the user accepts the cookie consent of this group.', 'mpp-cookieplus' ),
            ) );
            $xbox->add_field( array(
                'id' => 'group-' . $group_id . '-blocked-cookies',
                'name' => __( 'Blocked cookie names', 'mpp-cookieplus' ),
                'desc' => __( 'Enter the cookie names you want to block (comma separated)', 'mpp-cookieplus' ),
                'type' => 'textarea',
                'default' => self::$defaults['group-' . $group_id . '-blocked-cookies'],
                'attributes' => array(
                    'style' => 'min-height: 100px; height: 100px; '
                )
            ) );
            $xbox->add_field( array(
                'id' => 'group-' . $group_id . '-blocked-service-cookies',
                'name' => __( 'Blocked service cookies', 'mpp-cookieplus' ),
                'desc' => __( 'Automatically block cookies from the selected services.', 'mpp-cookieplus' ),
                'type' => 'checkbox',
                'default' => self::$defaults['group-' . $group_id . '-blocked-service-cookies'],
                'items' => CookieConsent::get_services(),
            ) );
        }

        $xbox->add_field( array(
            'type' => 'title',
            'name' => 'Scripts',
            'desc' => $title_scripts_desc,
        ) );
        $xbox->add_field( array(
            'id' => 'group-' . $group_id . '-header-scripts',
            'name' => 'Scripts in Head',
            'desc' => sprintf( __( 'Above script will be inserted into the %s section', 'mpp-cookieplus' ), '<code>&lt;head&gt;</code>' ),
            'type' => 'code_editor',
            'default' => self::$defaults['group-' . $group_id . '-header-scripts'],
            'options' => array(
                'language' => 'html',
                'theme' => 'tomorrow_night',
                'height' => '280px',
            ),
        ) );
        $xbox->add_field( array(
            'id' => 'group-' . $group_id . '-footer-scripts',
            'name' => 'Scripts in Footer',
            'desc' => sprintf( __( 'Above script will be inserted just before %s tag using %s hook', 'mpp-cookieplus' ), '<code>&lt;/body&gt;</code>', '<code>wp_footer</code>' ),
            'type' => 'code_editor',
            'default' => self::$defaults['group-' . $group_id . '-footer-scripts'],
            'options' => array(
                'language' => 'html',
                'theme' => 'tomorrow_night',
                'height' => '280px',
            ),
        ) );
    }

    public static function integration_options_for_cookie_consent( $xbox, $integration_name, $integration_data ){
        $section = $xbox->add_section( array(
            'id' => 'section-' . $integration_name,
            'name' => $integration_data['title'],
            'options' => array(
                'toggle' => true,
                'toggle_default' => 'close',
            )
        ) );
        $section->open_mixed_field( array( 'name' => __( 'Enable integration', 'mpp-cookieplus' ) ) );
        $section->add_field( array(
            'id' => $integration_name . '-enabled',
            'type' => 'switcher',
            'default' => self::$defaults[$integration_name . '-enabled'],
        ) );
        $section->add_field( array(
            'id' => $integration_name . '-group',
            'name' => __( 'Add to group', 'mpp-cookieplus' ),
            'desc' => __( 'Choose the group to which you want to assign this integration. If you choose the first one, it will be blocked until the user accepts the cookies independently of the accepted groups.', 'mpp-cookieplus' ),
            'type' => 'select',
            'default' => self::$defaults[$integration_name . '-group'],
            'items' => array(
                'block-until-accept' => __( 'Block until the user accepts cookies.', 'mpp-cookieplus' ),
                'group_1' => __( 'Group 1', 'mpp-cookieplus' ) . ' - ' . Options::option( 'group-1-title' ),
                'group_2' => __( 'Group 2', 'mpp-cookieplus' ) . ' - ' . Options::option( 'group-2-title' ),
                'group_3' => __( 'Group 3', 'mpp-cookieplus' ) . ' - ' . Options::option( 'group-3-title' ),
                'group_4' => __( 'Group 4', 'mpp-cookieplus' ) . ' - ' . Options::option( 'group-4-title' ),
                'group_5' => __( 'Group 5', 'mpp-cookieplus' ) . ' - ' . Options::option( 'group-5-title' ),
            ),
            'grid' => '5-of-6'
        ) );
        $section->close_mixed_field();

        if( $integration_data['use_id'] ){
            $section->add_field( array(
                'id' => $integration_name . '-id',
                'name' => $integration_data['id_name'],
                'desc' => $integration_data['id_desc'],
                'type' => 'text',
                'default' => self::$defaults[$integration_name . '-id'],
                'grid' => '3-of-6',
            ) );
        }
        $section->open_mixed_field( array( 'name' => __( 'Code', 'mpp-cookieplus' ) ) );
        $section->add_field( array(
            'id' => $integration_name . '-code',
            'type' => 'code_editor',
            'default' => self::$defaults[$integration_name . '-code'],
            'options' => array(
                'language' => 'html',
                'theme' => 'tomorrow_night',
                'height' => '200px',
                'show_name' => false
            ),
        ) );
        $section->add_field( array(
            'id' => $integration_name . '-in-footer',
            'type' => 'checkbox',
            'default' => self::$defaults[$integration_name . '-in-footer'],
            'items' => array(
                '1' => 'Load in footer',
            ),
            'options' => array(
                'show_name' => false
            ),
        ) );
        $section->close_mixed_field();
    }

}