<?php namespace MasterPopupsAddonCookiePlus\Includes;

use GeoIp2\Database\Reader;

class SettingsPanel {
    protected static $instance = null;
    public $plugin;
    public static $defaults = array();
    public static $defaults_popup_editor = array();

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Debug
    |---------------------------------------------------------------------------------------------------
    */
    public function d( $var1, $var2 = null ){
        return $this->plugin->d( $var1, $var2 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_action( 'xbox_init', array( $this, 'create_metabox_admin_page' ) );
        add_action( 'init', array( $this, 'init' ), 1 );

        add_filter( "mpp_popup_options", array( $this, 'set_popup_options' ), 10, 2 );
        add_filter( "mpp_public_popup_options", array( $this, 'set_public_popup_options' ), 10, 2 );
        add_filter( "mpp_element_data", array( $this, 'set_element_data' ), 10, 2 );
        add_filter( "mpp_should_display_popup", array( $this, 'should_display_popup_on_countries' ), 10, 2 );

        add_filter( 'mpp_popup_editor_tab_items', array( $this, 'add_item_to_popup_editor' ), 10, 2 );
        add_filter( 'mpp_popup_editor_tab_fields', array( $this, 'add_fields_to_popup_editor' ), 10, 1 );

        add_filter( 'mpp_elements_tab_items', array( $this, 'add_item_to_elements' ), 10, 3 );
        add_filter( 'mpp_elements_tab_fields', array( $this, 'add_fields_to_elements' ), 10, 2 );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Init action
    |---------------------------------------------------------------------------------------------------
    */
    public function init(){
        $this->set_default_options();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Establece las opciones por defecto
    |---------------------------------------------------------------------------------------------------
    */
    public function set_default_options( $options = array() ){
        $defaults = include dirname( __FILE__ ) . '/options/defaults.php';

        self::$defaults = wp_parse_args( $options, $defaults['cookieplus'] );
        self::$defaults_popup_editor = $defaults['masterpopups']['popup_editor'];
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega opciones al popup y verifica el uso de Cookie Settings
	|---------------------------------------------------------------------------------------------------
	*/
    public function set_popup_options( $options, $popup ){
        $new_options = array();
        foreach( self::$defaults_popup_editor as $name => $value ){
            $popup->set_option_to( $new_options, $name, $value );
        }
        $options = wp_parse_args( $new_options, $options );
        if( $options['mpp_cookieplus-use-has-cookie-settings'] == 'on' ){
            $options['mpp_use-wp-editor'] = 'on';
        }
        return $options;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega opciones para el plugin js
	|---------------------------------------------------------------------------------------------------
	*/
    public function set_public_popup_options( $options, $popup ){
        $options['cookieplus'] = array(
            'use_has_cookie_settings' => $popup->option( 'cookieplus-use-has-cookie-settings' ) == 'on' ? true : false,
            'actions' => array(
                'on_close_popup' => $popup->option( 'cookieplus-action-on-close-popup' ),
            ),
            'show_popup_after_accept_cookies' => $popup->option( 'cookieplus-show-popup-after-accept-cookies' ) == 'on' ? true : false,
            'show_popup_after_decline_cookies' => $popup->option( 'cookieplus-show-popup-after-decline-cookies' ) == 'on' ? true : false,
            'disclaimer_enabled' => $popup->option( 'cookieplus-disclaimer-enabled' ) == 'on' ? true : false,
        );
        return $options;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega opciones al atributo data de cada elemento del popup
	|---------------------------------------------------------------------------------------------------
	*/
    public function set_element_data( $element_data, $element ){
        $new_data = array(
            'actions' => array(
                'on_click' => $element->option( 'cookieplus-on-click-element' ),
            )
        );
        $new_data = json_encode( $new_data );
        $element_data['cookieplus'] = $new_data;
        return $element_data;
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Geo info
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_my_geo_info(){
        $ip = $this->plugin->get_real_or_fake_ip();
        return $this->plugin->get_geo_info( $ip );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si se debe mostrar el popup en determinados países
	|---------------------------------------------------------------------------------------------------
	*/
    public function should_display_popup_on_countries( $display, $popup ){
        if( $display ){
            $show_on_countries = $popup->option( 'cookieplus-show-on-countries' );
            $hide_on_countries = $popup->option( 'cookieplus-hide-on-countries' );

            if( $show_on_countries == 'on' || $hide_on_countries == 'on' ){
                $geo_info = $this->get_my_geo_info();
                if( ! $geo_info ){
                    return $display;
                }

                if( $show_on_countries == 'on' ){
                    $show_countries = $popup->option( 'cookieplus-show-countries' );
                    if( $popup->option( 'cookieplus-show-on-eu-countries' ) == 'on' ){
                        $show_countries = array_merge( $show_countries, array_keys( \XboxItems::eu_countries() ) );
                    }
                    if( in_array( $geo_info->country->isoCode, $show_countries ) ){
                        $display = true;
                    } else{
                        $display = false;
                    }
                }

                if( $display && $hide_on_countries == 'on' ){
                    $hide_countries = $popup->option( 'cookieplus-hide-countries' );
                    if( $popup->option( 'cookieplus-hide-on-eu-countries' ) == 'on' ){
                        $hide_countries = array_merge( $hide_countries, array_keys( \XboxItems::eu_countries() ) );
                    }
                    if( in_array( $geo_info->country->isoCode, $hide_countries ) ){
                        $display = false;
                    } else{
                        $display = true;
                    }
                }
            }
        }
        return $display;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Crea página principal de opciones
	|---------------------------------------------------------------------------------------------------
	*/
    public function create_metabox_admin_page( $hook ){
        $options = array(
            'id' => $this->plugin->arg( 'settings_id' ),
            'title' => $this->plugin->arg( 'menu_title' ),
            'menu_title' => $this->plugin->arg( 'menu_title' ),
            'parent' => 'edit.php?post_type=' . $this->plugin->arg( 'masterpopups', 'post_type' ),
            'capability' => 'manage_options',
            'header' => array(
                'desc' => 'GDPR Cookie Consent Solution',
                'icon' => '<img src="' . CPMPP_URL . 'assets/admin/images/icon-cookieplus-white.png">',
                'class' => 'cookieplus-header-icon',
                'submit-buttons-sticky' => true,
            ),
            'class' => 'ampp',
            'footer' => $this->plugin->arg( 'menu_title' ) . ' v' . CPMPP_VERSION
        );
        $xbox = xbox_new_admin_page( $options );
        $wp_pages = \XboxItems::posts_by_post_type( 'page', array( 'posts_per_page' => -1 ) );

        include CPMPP_DIR . 'includes/options/settings-panel/settings-panel.php';
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Options for Popup Editor main tab
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_item_to_popup_editor( $items, $id_tab ){
        $items['cookieplus'] = '<img src="' . CPMPP_URL . 'assets/admin/images/icon-cookieplus-32.png">Cookie Plus';
        return $items;
    }

    public function add_fields_to_popup_editor( $xbox ){
        if( $xbox ){
            $xbox->open_tab_item( 'cookieplus' );
            include CPMPP_DIR . 'includes/options/popup-editor/popup-editor.php';
            $xbox->close_tab_item( 'cookieplus' );
        }
        return $xbox;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Add item to elements tab
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_item_to_elements( $items, $id_tab, $device ){
        $items[$device . '-elements-cookieplus'] = '<img src="' . CPMPP_URL . 'assets/admin/images/icon-cookieplus-32.png">Cookie Plus';
        return $items;
    }

    public function add_fields_to_elements( $elements, $device ){
        if( $elements ){
            $elements->open_tab_item( $device . '-elements-cookieplus' );
            include CPMPP_DIR . 'includes/options/popup-editor/elements-tab.php';
            $elements->close_tab_item( $device . '-elements-cookieplus' );
        }
        return $elements;
    }


}