<?php namespace MasterPopupsAddonCookiePlus\Includes;

use Xbox\Includes\CSS;


class TabCookieSettings {
    protected static $instance = null;
    public $plugin;

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->hooks();

        //Evita el error: Fatal error: Class 'Xbox\Includes\CSS' not found.
        //Este error sucede en algunas sitios cuando no se ha cargado Xbox porque xbox_init se llama demasiado tarde
        $loader = \MasterPopupsAddonCookiePlus\Loader::get_instance();
        $plugin_data = $loader->get_plugin_data();
        if( ! empty( $plugin_data ) && ! empty( $plugin_data['basename'] ) ){
            $masterpopups_path = trailingslashit( WP_PLUGIN_DIR ) . explode( '/', $plugin_data['basename'])[0];
            $css_file_path = trailingslashit( $masterpopups_path ) . 'libs/xbox/includes/class-css.php';
            if( ! class_exists( '\Xbox\Includes\CSS' ) && is_dir( $css_file_path ) ){
                include_once trailingslashit( $masterpopups_path ) . 'libs/xbox/includes/class-css.php';
            }
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_filter( "mpp_popup_content", array( $this, 'build' ), 10, 2 );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Debug
    |---------------------------------------------------------------------------------------------------
    */
    public function d( $var1, $var2 = null ){
        return $this->plugin->d( $var1, $var2 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Cookie Settings
	|---------------------------------------------------------------------------------------------------
	*/
    public function build( $content, $popup ){
        if( $popup->option( 'cookieplus-use-has-cookie-settings' ) != 'on' ){
            return $content;
        }

        if( ! $this->plugin->CookieConsent->is_enabled() ){
            $error_message = __( 'Cookie Settings is not available. Cookie Consent is disabled or is just disabled for your country.', 'mpp-cookieplus' );
            return $content.$error_message;
        }

        $presentation = Options::option( 'presentation' );
        $theme = 'cookieplus-'.Options::option( 'theme' );
        $logo_img = '';
        $default_logo = Options::option( 'default-logo' );
        if( $default_logo == 'logo-upload.png' ){
            $logo_url = Options::option( 'custom-logo' );
        } else {
            $logo_url = CPMPP_URL.'assets/public/images/'.$default_logo;
        }
        $class = "cookieplus-cookie-consent $theme cookieplus-$presentation ";
        if( $logo_url ){
            $class .= ' has-logo';
            $logo_img = '<img src="'.$logo_url.'">';
        }

        $cookies_accepted = $this->plugin->CookieConsent->cookies_accepted();

        $cookie_settings = "<div class='{$class}' data-popup-id='{$popup->id}' data-cookies-accepted='$cookies_accepted' data-presentation='{$presentation}'>";
        $cookie_settings .= $this->build_header( $popup, $logo_img, $logo_url );
        $cookie_settings .= $this->build_body( $popup );

        $cookie_settings .= $this->build_footer();
        $cookie_settings .= '</div>';
        $cookie_settings = stripslashes( $cookie_settings );//Evita slash en \'

        $style = "\n<style>\n";
        $style .= $this->build_popup_style( $popup->id );
        $style .= "\n</style>\n";

        return $cookie_settings.$content.$this->get_link_powered_by().$style;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Header Cookie Settings
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_header( $popup, $logo_img, $logo_url ){
        $header = '';
        $header .= "<div class='cookieplus-header-cookie-consent'>";

        if( $logo_url ){
            $header .= '<div class="cookieplus-logo-cookie-consent">';
            $header .= $logo_img;
            $header .= '</div>';//.cookieplus-logo-cookie-consent
        }
        $header .= '<div class="cookieplus-info-cookie-consent">';
        $title = Options::option( 'cookie-settings-title' );
        if( Functions::starts_with('<', $title) || Functions::ends_with('>', $title) ){
            $header .= $title;
        } else {
            $header .= '<h2>'.$title.'</h2>';
        }
        $header .= '</div>';//.cookieplus-info-cookie-consent
        $header .= '</div>';//.cookieplus-header-cookie-consent
        return $header;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Body Cookie Settings
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_body( $popup ){
        $body = '';
        $presentation = Options::option( 'presentation' );
        $body .= "<div class='cookieplus-body-cookie-consent'>";

        if( $presentation == 'compact' || $presentation == 'combined' ){
            $body .= $this->parse_content( Options::option( 'cookie-settings-content' ) );
            foreach( $this->plugin->CookieConsent->get_enabled_groups() as $i => $group_id ){
                $group_title = Options::option( 'group-' . $i . '-title' );
                $body .= $this->build_switcher_cookie_group( $i, $group_title );
            }
        }

        if( $presentation == 'combined' ){
            $show_text = Options::option( 'show-cookie-tabs-text' );
            $body .= '<div class="cookieplus-toggle-tabs"><a href="#">'.$show_text.'</a></div>';
        }

        if( $presentation == 'extended' || $presentation == 'combined' ){
            $body .= $this->build_tab( $popup );
        }
        $body .= '</div>';//.cookieplus-body-cookie-consent

        return $body;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Tab Cookie Settings
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_tab( $popup ){
        $tab_items = '';
        $tab_panels = '';
        foreach( $this->plugin->CookieConsent->get_enabled_groups() as $i => $group_id ){
            $tab_items .= $this->build_tab_item( $i, $popup->id );
            $tab_panels .= $this->build_panel_item( $i, $popup->id );
        }


        $presentation = Options::option( 'presentation' );

        $first_tab = '';
        $first_panel = '';
        if( $presentation == 'extended' ){
            $first_tab = $this->build_custom_tab_item( Options::option( 'cookie-overview-tab-text' ), 'overview', $popup->id );
            $first_panel = $this->build_custom_panel_item( Options::option( 'cookie-settings-content' ), 'overview', $popup->id);
        }

        $cookie_policy_tab = '';
        $url = Options::option( 'cookie-policy-tab-url' );
        if( Options::option( 'show-cookie-policy-tab' ) == 'on' && $url != '' ){
            $cookie_policy_tab .= $this->build_custom_tab_item( Options::option( 'cookie-policy-tab-text' ), $url);
        }

        $tabster_tabs = '<ul class="tabster-tab-group" role="tablist">';
        $tabster_tabs .= $first_tab.$tab_items . $cookie_policy_tab;
        $tabster_tabs .= '</ul>';

        $tabster_panels = '<div class="tabster-panel-group">';
        $tabster_panels .= $first_panel;
        $tabster_panels .= $tab_panels;
        $tabster_panels .= '</div>';

        $tabster = '<div class="cookieplus-wrap-tabs-cookie-consent"><div class="cookieplus-tabs-cookie-consent">';
        $tabster .= $tabster_tabs . $tabster_panels;
        $tabster .= '</div></div>';

        return $tabster;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Parse text content
	|---------------------------------------------------------------------------------------------------
	*/
    public function parse_content( $content ){
        $content = do_shortcode( $content );
        $content = wpautop( $content );
        return $content;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Tab item
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_tab_item( $group_id, $popup_id ){
        $tab_item = '';
        $tab_item .= '<li role="presentation" class="tabster-tab-item">';
        $tab_item .= '<a role="tab" class="tabster-tab" href="#tabsterItem_' . $group_id . '_popup_' . $popup_id . '">';
        $tab_item .= Options::option( 'group-' . $group_id . '-title' );
        $tab_item .= '</a>';
        $tab_item .= '</li>';
        return $tab_item;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Panel item
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_panel_item( $group_id, $popup_id ){
        $tab_panel = '';
        $tab_panel .= '<div role="tabpanel" id="tabsterItem_' . $group_id . '_popup_' . $popup_id . '" class="tabster-panel" data-group="' . $group_id . '">';
        $group_content = Options::option( 'group-' . $group_id . '-content' );
        $group_content = $this->parse_content( $group_content );
        $tab_panel .= $group_content;

        $presentation = Options::option( 'presentation' );
        if( $presentation == 'extended' ){
            $tab_panel .= $this->build_switcher_cookie_group( $group_id );
        }

        $tab_panel .= '</div>';

        return $tab_panel;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Custom Tab item
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_custom_tab_item( $title, $href, $popup_id = null ){
        $tab_item = '';
        $tab_item .= '<li role="presentation" class="tabster-tab-item">';
        $link = '<a role="tab" class="tabster-tab" href="' . $href . '" target="_blank">';
        if( $popup_id && $popup_id != null ){
            $link = '<a role="tab" class="tabster-tab" href="#tabsterItem_' . $href . '_popup_' . $popup_id . '">';
        }
        $tab_item .= $link;
        $tab_item .= $title;
        $tab_item .= '</a>';
        $tab_item .= '</li>';
        return $tab_item;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Custom Panel item
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_custom_panel_item( $content, $id, $popup_id ){
        $tab_panel = '';
        $tab_panel .= '<div role="tabpanel" id="tabsterItem_' . $id . '_popup_' . $popup_id . '" class="tabster-panel">';
        $content = $this->parse_content( $content );
        $tab_panel .= $content;
        $tab_panel .= '</div>';
        return $tab_panel;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build Switcher
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_switcher_cookie_group( $group_id, $group_title = null ){
        $consent_mode = Options::option( 'cookie-consent-mode' );
        if( $consent_mode == 'block-all' ){
            return '';
        }

        $consent_cookie = $this->plugin->CookieConsent->is_consent_given_to_group( $group_id );

        if( $group_id == 1 ){
            $text = is_null( $group_title ) ? Options::option( 'texts-cookie-always-active' ) : $group_title;
            $info = '<span class="cookieplus-switcher-info">' . $text . '</span>';
            $input = "<input type='hidden' class='cookieplus-switcher-input' value='on' data-group='$group_id' disabled>";
        } else if( $consent_cookie ){
            $text = is_null( $group_title ) ? Options::option( 'texts-cookie-on' ) : $group_title;
            $info = '<span class="cookieplus-switcher-info">' . $text . '</span>';
            $input = "<input type='hidden' class='cookieplus-switcher-input' value='on' data-group='$group_id' data-consent='$consent_cookie'>";
        } else{
            $text = is_null( $group_title ) ? Options::option( 'texts-cookie-off' ) : $group_title;
            $info = '<span class="cookieplus-switcher-info is-disabled">' . $text . '</span>';
            $input = "<input type='hidden' class='cookieplus-switcher-input' value='off' data-group='$group_id' data-consent='$consent_cookie'>";
        }

        $switcher = "<div class='status-cookie-group'>";
        $switcher .= "<div class='cookieplus-switcher'>";
        $switcher .= $input . $info;
        $switcher .= "</div>";
        $switcher .= "</div>";
        return $switcher;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build footer
    |---------------------------------------------------------------------------------------------------
    */
    public function build_footer(){
        $return = '';
        $accep_cookies_text = Options::option( 'btn-accept-cookies-text' );
        $save_settings_text = Options::option( 'btn-save-settings-text' );
        $decline_cookies_text = Options::option( 'btn-decline-cookies-text' );
        $btn_class = 'cookieplus-btn cookieplus-btn-green';

        $return .= '<div class="cookieplus-footer-cookie-consent">';
        $return .= '<a href="#" class="' . $btn_class . ' cookieplus-btn-accept-cookies">' . $accep_cookies_text . '</a>';
        if( Options::option( 'cookie-consent-mode' ) == 'by-groups' ){
            $return .= '<a href="#" class="' . $btn_class . ' cookieplus-btn-save-settings">' . $save_settings_text . '</a>';
        }
        if( Options::option( 'btn-decline-cookies-enabled' ) == 'on' ){
            $return .= '<a href="#" class="cookieplus-btn cookieplus-btn-red cookieplus-btn-decline-cookies">' . $decline_cookies_text . '</a>';
        }

        $return .= '</div>';
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Link powered by
    |---------------------------------------------------------------------------------------------------
    */
    public function get_link_powered_by(){
//        $return = '<div class="cookieplus-wrap-link-powered-by">';
//        if( Options::option( 'link-powered-by-enabled' ) == 'on' ){
//            $username = Options::option( 'link-powered-by-username' );
//            if( empty( $username ) ){
//                $username = 'codexhelp';
//            }
//            $link_powered_by = "https://codecanyon.net/item/cookie-plus-gdpr-cookie-consent-solution-for-wordpress/21984547?ref=$username";
//            $text_powered_by = "Powered by <span>Cookie Plus</span>";
//            $return .= '<a href="' . $link_powered_by . '" target="_blank" class="cookieplus-link-powered-by">' . $text_powered_by . '</a>';
//        }
//        return $return.'</div>';
        return '';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build popup style
    |---------------------------------------------------------------------------------------------------
    */
    public function build_popup_style( $popup_id ){
        $style = '';
        $theme = Options::option( 'theme' );
        $padding_right_logo = 30;
        $menu_width = (int) Options::option( 'tab-menu-width' ) + $padding_right_logo;
        $padding_x = 40;

        $css = new CSS( ".mpp-wrap-$popup_id .mpp-content" );
        $css->prop( 'padding', '0 !important' );
        $style .= $css->build_css();

        if( $theme == 'modern'){
            $css = new CSS( ".mpp-wrap-$popup_id .cookieplus-cookie-consent.cookieplus-modern" );
            $css->prop( 'padding', "20px {$padding_x}px 10px {$padding_x}px !important" );
            $style .= $css->build_css();

            $css = new CSS( ".cookieplus-cookie-consent.cookieplus-modern .cookieplus-logo-cookie-consent" );
            $css->prop( 'width', "{$menu_width}px" );
            $css->prop( 'padding-right', "{$padding_right_logo}px" );
            $style .= $css->build_css();

            $css = new CSS( ".cookieplus-cookie-consent.cookieplus-modern:after" );
            $css->prop( 'left', ($menu_width + $padding_x).'px' );
            $css->prop( 'background-color', Options::option( 'tab-content-bg-modern' ) );
            $style .= $css->build_css();

            $css = new CSS( ".cookieplus-cookie-consent.cookieplus-modern .tabster-vertical .tabster-panel-group" );
            $css->prop( 'background-color', Options::option( 'tab-content-bg-modern' ) );
            $style .= $css->build_css();

            $css = new CSS( ".cookieplus-cookie-consent.cookieplus-modern .tabster-tab-item" );
            $css->prop( 'border-radius', CSS::number( Options::option( 'tab-item-border-radius' ), 'px' ) );
            $style .= $css->build_css();
        }

        $css = new CSS( ".mpp-wrap-$popup_id .mpp-content-wp-editor" );
        $css->prop( 'padding', "0px !important" );
        $css->prop( 'display', "block !important" );
        $style .= $css->build_css();

        $css = new CSS( ".mpp-wrap-$popup_id .mpp-close-icon" );
        $css->prop( 'z-index', '20' );
        $style .= $css->build_css();

        $css = new CSS( ".mpp-wrap-$popup_id .cookieplus-cookie-consent.cookieplus-classic" );
        $css->prop( 'padding', "30px {$padding_x}px 10px {$padding_x}px !important" );
        $style .= $css->build_css();

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build style
    |---------------------------------------------------------------------------------------------------
    */
    public function build_style(){
        $style = '';
        $root = '.cookieplus-cookie-consent';
        $css = new CSS( $root );
        $line_height = (float) Options::option( 'tab-container-line-height' );
        $css->prop( 'font-size', CSS::number( Options::option( 'tab-container-font-size' ), 'px' ) );
        $css->prop( 'line-height', CSS::number( $line_height, 'em' ) );

        //Google font
        if( Options::option( 'use-font-family' ) == 'on' ){
            $css->prop( 'font-family', Options::option( 'font-family' ) );
        }

        $style .= $css->build_css();

        $css = new CSS( "$root .cookieplus-info-cookie-consent *" );
        $css->prop( 'line-height', CSS::number( ($line_height - 0.4), 'em' ) );
        $style .= $css->build_css();

        $_tab = '.cookieplus-tabs-cookie-consent';
        $css = new CSS( "$_tab" );
        $css->prop( 'font-size', '1em' );
        $css->prop( 'background', Options::option( 'tab-container-bg' ) );
        $css->prop( 'border-color', Options::option( 'tab-container-border-color' ) );
        $css->prop( 'border-radius', CSS::number( Options::option( 'tab-container-border-radius' ), 'px' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab.tabster-vertical .tabster-tab-group, .cookieplus-logo-cookie-consent" );
        $css->prop( 'width', CSS::number( Options::option( 'tab-menu-width' ), 'px' ) );
        $style .= $css->build_css();

        $css = new CSS( ".cookieplus-logo-cookie-consent" );
        $css->prop( 'width', CSS::number( Options::option( 'tab-menu-width' ), 'px' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-tab-item, $_tab .tabster-panel-heading" );
        $css->prop( 'border-color', Options::option( 'tab-item-border-color' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-tab, $_tab .tabster-panel-heading" );
        $css->prop( 'background', Options::option( 'tab-item-bg' ) );
        $css->prop( 'color', Options::option( 'tab-item-color' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-tab:hover, $_tab .tabster-panel-heading:hover" );
        $css->prop( 'color', Options::option( 'tab-item-color-hover' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-tab-item.is-active > .tabster-tab, $_tab .tabster-panel-heading.is-active > .tabster-tab" );
        $css->prop( 'background', Options::option( 'tab-item-active-bg' ) );
        $css->prop( 'color', Options::option( 'tab-item-active-color' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-tab-item.is-active, $_tab .tabster-panel-heading.is-active" );
        $css->prop( 'border-color', Options::option( 'tab-item-active-border-color' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-tab-item.is-active:after" );
        $css->prop( 'background', Options::option( 'tab-item-active-bg' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-panel-group" );
        $css->prop( 'background', Options::option( 'tab-content-bg' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab.tabster-vertical .tabster-panel-group" );
        $css->prop( 'border-color', Options::option( 'tab-item-border-color' ) );
        $style .= $css->build_css();

        $css = new CSS( "$_tab .tabster-panel" );
        $css->prop( 'padding', Options::option( 'tab-content-padding' ) );
        $style .= $css->build_css();

        $css = new CSS( "$root .cookieplus-switcher .cookieplus-switcher-info" );
        $css->prop( 'color', Options::option( 'color-cookie-group-on' ) );
        $style .= $css->build_css();

        $css = new CSS( "$root .cookieplus-switcher .cookieplus-switcher-info.is-disabled" );
        $css->prop( 'color', Options::option( 'color-cookie-group-off' ) );
        $style .= $css->build_css();

        $style .= $this->build_footer_style();

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build footer style
    |---------------------------------------------------------------------------------------------------
    */
    public function build_footer_style(){
        $style = '';
        $css = new CSS( ".cookieplus-btn.cookieplus-btn-accept-cookies" );
        $css->prop( 'background', Options::option( 'btn-accept-cookies-bg' ) );
        $css->prop( 'color', Options::option( 'btn-accept-cookies-color' ) );
        $style .= $css->build_css();

        $css = new CSS( ".cookieplus-btn.cookieplus-btn-accept-cookies:hover" );
        $css->prop( 'background', Options::option( 'btn-accept-cookies-bg-hover' ) );
        $style .= $css->build_css();

        $css = new CSS( ".cookieplus-btn.cookieplus-btn-save-settings" );
        $css->prop( 'background', Options::option( 'btn-save-settings-bg' ) );
        $css->prop( 'color', Options::option( 'btn-save-settings-color' ) );
        $style .= $css->build_css();

        $css = new CSS( ".cookieplus-btn.cookieplus-btn-save-settings:hover" );
        $css->prop( 'background', Options::option( 'btn-save-settings-bg-hover' ) );
        $style .= $css->build_css();

        $css = new CSS( ".cookieplus-btn.cookieplus-btn-decline-cookies" );
        $css->prop( 'background', Options::option( 'btn-decline-cookies-bg' ) );
        $css->prop( 'color', Options::option( 'btn-decline-cookies-color' ) );
        $style .= $css->build_css();

        $css = new CSS( ".cookieplus-btn.cookieplus-btn-decline-cookies:hover" );
        $css->prop( 'background', Options::option( 'btn-decline-cookies-bg-hover' ) );
        $style .= $css->build_css();

        $css = new CSS( ".cookieplus-footer-cookie-consent .cookieplus-btn" );
        $css->prop( 'border-radius', CSS::number( Options::option( 'tab-container-border-radius' ), 'px' ) );
        $style .= $css->build_css();

        return $style;
    }

}