<?php namespace MasterPopupsAddonCookiePlus\Includes\Integrations;

use MasterPopupsAddonCookiePlus\Includes\Options;
use MasterPopupsAddonCookiePlus\Includes\Functions;

class CF7 {
    protected static $instance = null;
    public $plugin;
    private $privacy_policy_name = 'accept_privacy_policy';

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if ( null === self::$instance ) {
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    protected function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->hooks();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_filter( 'wpcf7_form_elements', array( $this, 'add_privacy_policy_checkbox' ), 10 );//do_shortcode the_content hook 10
        add_filter( 'wpcf7_validate', array( $this, 'validate_privacy_policy_checkbox' ), 10, 2 );//parse_request 20
        add_action( 'wpcf7_init', array( $this, 'create_tag_privacy_policy_checkbox' ), 10 );//init hook 10
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Checkbox
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_checkbox( $class = '' ){
        $checkbox = '';
        $text = Options::option( 'cf7-policy-text' );
        $page_id = Options::option( 'privacy-policy-page' );
        if ( $page_id > 0 ) {
            $text = '<a href="' . Functions::get_permalink( $page_id ) . '" target="_blank">' . $text . '</a>';
        }
        $checkbox .= '<p class="cookieplus">';
        $checkbox .= '<span class="wpcf7-form-control-wrap ' . $this->privacy_policy_name . '">';
        $checkbox .= '<span class="wpcf7-form-control wpcf7-checkbox wpcf7-validates-as-required">';
        $checkbox .= '<label class="cookieplus-privacy-policy-label" style="cursor: pointer;">';
        $checkbox .= '<input type="checkbox" name="' . $this->privacy_policy_name . '" class="' . $class . '" required="required" aria-required="true" aria-invalid="false">';
        $checkbox .= $text . ' <span class="required">*</span>';
        $checkbox .= '</label>';
        $checkbox .= '</span></span>';
        $checkbox .= '</p>';

        return $checkbox;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega checkbox automáticamente al formulario de contacto de Contact Form 7
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_privacy_policy_checkbox( $html ){
        if ( Options::option( 'cf7-enabled' ) == 'on' && Options::option( 'cf7-tag-auto' ) == 'on' ) {
            if ( stripos( $html, 'cookieplus-privacy-policy-label' ) === false ) {
                $checkbox = $this->get_checkbox();
                return $html . $checkbox;
            }
        }
        return $html;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Crea tag para CookiePlus en Contact Form 7
	|---------------------------------------------------------------------------------------------------
	*/
    public function create_tag_privacy_policy_checkbox(){
        $tag_name = 'cookieplus_checkbox_accept_policy';
        wpcf7_add_form_tag(
            array( $tag_name, $tag_name . '*' ),
            array( $this, 'cookieplus_checkbox_accept_policy_tag_handler' ),
            array( 'name-attr' => true )
        );
    }
    public function cookieplus_checkbox_accept_policy_tag_handler( $tag ){
        if ( Options::option( 'cf7-enabled' ) == 'on' ){
            return $this->get_checkbox();
        }
        return '';
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Valida checkbox políticas de privacidad en Contact Form 7
	|---------------------------------------------------------------------------------------------------
	*/
    public function validate_privacy_policy_checkbox( $result, $tags ){
        if ( Options::option( 'cf7-enabled' ) == 'on' ){
            $tag = new \WPCF7_FormTag();
            $tag->name = $this->privacy_policy_name;
            if( ! isset( $_POST[$tag->name] ) ){
                $result->invalidate( $tag, Options::option( 'cf7-policy-validation-text' ) );
            }
        }
        return $result;
    }

}