<?php

use MasterPopupsAddonCookiePlus\Includes\Functions;
use MasterPopupsAddonCookiePlus\Includes\Options;
use MasterPopupsAddonCookiePlus\Includes\BrowserDetection;

$images_url = CPMPP_URL . 'assets/admin/images/';
$plugin = Functions::get_plugin_instance();
$prefix = $plugin->arg( 'prefix' );

$total_consent_logs = wp_count_posts( $plugin->post_types['consent_logs'] );
$total_consent_logs = $total_consent_logs ? $total_consent_logs->publish : 0;

function cookieplus_get_consent_logs_table( $count = 100 ){
    //LLenar tabla solo en el panel de opciones
    $post_type = \MasterPopups\Includes\Functions::get_current_post_type();
    if( ! in_array( $post_type, array( 'master-popups' ) ) ){
        return '';
    }

    $return = '';
    $plugin = Functions::get_plugin_instance();
    $consent_logs = $plugin->get_consent_logs( array( 'numberposts' => $count ) );

    if( Functions::is_empty( $consent_logs ) ){
        return $return;
    }

    $return .= "<table id='cpp-table-cookie-logs-overview' class='ampp-table'>";
    $return .= "<thead><tr>";
    $return .= "<th class='noHide'>" . __( 'Consent Type', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th class='noHide'>" . __( 'Status', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Accepted All', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Accepted Cookies', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Declined Cookies', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'User ID', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Country', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'IP Address', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Platform', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Browser', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Device', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Page URL', 'mpp-cookieplus' ) . "</th>";
    //    $return .= "<th>" . __( 'Cookie Name', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th>" . __( 'Expiration', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th><i class='xbox-icon xbox-icon-calendar'></i> " . __( 'Date Created', 'mpp-cookieplus' ) . "</th>";
    $return .= "<th class='noHide'><input type='checkbox' class='ampp-checkbox-all-logs'><a href='#' class='ampp-delete-all-logs' title='Delete'><i class='xbox-icon xbox-icon-trash xbox-color-red'></i></a></th>";
    $return .= "</tr></thead><tbody>";

    foreach( $consent_logs as $log ){
        $type = get_post_meta( $log->ID, 'cpp_type', true );
        $version = get_post_meta( $log->ID, 'cpp_version', true );
        $accepted = get_post_meta( $log->ID, 'cpp_accepted', true );
        $declined = get_post_meta( $log->ID, 'cpp_declined', true );
        //$cookie_name = get_post_meta( $log->ID, 'cpp_cookie_name', true );
        $expiration_time = get_post_meta( $log->ID, 'cpp_expiration_time', true );
        $date_created = get_post_meta( $log->ID, 'cpp_date_created', true );
        $groups = get_post_meta( $log->ID, 'cpp_groups', true );
        $groups_info = get_post_meta( $log->ID, 'cpp_groups_info', true );
        $user_id = get_post_meta( $log->ID, 'cpp_user_id', true );
        $user_ip = get_post_meta( $log->ID, 'cpp_user_ip', true );
        $user_agent = get_post_meta( $log->ID, 'cpp_user_agent', true );
        $browser = BrowserDetection::get_browser( $user_agent );
        $platform = BrowserDetection::get_platform( $user_agent );
        $desktop_or_mobile = BrowserDetection::is_mobile( $user_agent ) ? 'Mobile' : 'Desktop';
        $page_url = get_post_meta( $log->ID, 'cpp_page_url', true );

        //Versión para: Privacy Policy & Terms & Conditions
        $version_info = '';
        if( $type == CPMPP_KEY_TERMS_CONSENT || $type == CPMPP_KEY_PRIVACY_CONSENT ){
            $version_info = 'v' . $version;
        }

        //Consent Type
        $accepted_text = $declined ? 'Declined' : 'Accepted';
        $status_class = $accepted ? 'cpp-accepted' : 'cpp-declined';
        $consent_status = "<span class='cpp-small cpp-small-bg $status_class'>$accepted_text $version_info</span>";
        $title_array = array(
            CPMPP_KEY_COOKIE_CONSENT => __( 'Cookies Consent', 'mpp-cookieplus' ),
            CPMPP_KEY_TERMS_CONSENT => __( 'Terms & Conditions', 'mpp-cookieplus' ),
            CPMPP_KEY_PRIVACY_CONSENT => __( 'Privacy Policy', 'mpp-cookieplus' ),
        );
        $consent_type = isset( $title_array[$type] ) ? $title_array[$type] : '';

        //Consent info
        $accepted_cookies = '';
        $declined_cookies = '';
        $accepted_all = ( $groups == 'all' || $groups == '' || $groups == null ) ? __( 'Yes', 'mpp-cookieplus' ) : __( 'No', 'mpp-cookieplus' );
        if( $type == CPMPP_KEY_COOKIE_CONSENT ){
            $groups_info = ! is_array( $groups_info ) ? array() : $groups_info;
            foreach( $groups_info as $info ){
                $status_class = $info['consent_given'] == 'true' ? 'cpp-accepted' : 'cpp-declined';
                if( $info['consent_given'] == 'true' ){
                    $accepted_cookies .= "<span class='cpp-small cpp-small-normal $status_class'>{$info['title']}</span>,<br>";
                } else{
                    $declined_cookies .= "<span class='cpp-small cpp-small-normal $status_class'>{$info['title']}</span>,<br>";
                }
            }
            $accepted_cookies = rtrim( $accepted_cookies, ',<br>' );
            $declined_cookies = rtrim( $declined_cookies, ',<br>' );
        }

        //Expiration
        if( $expiration_time ){
            $datetime = new DateTime( "@$expiration_time" );
            $datetime->setTimeZone( Functions::get_timezone() );//wordpress timezone
            $expiration_time = $datetime->format( "m/d/Y h:i:s a" );
        }

        //Date Created
        if( $date_created ){
            $date_created = date( "j M Y, g:i a", strtotime( $date_created ) );//date_created ya tiene wordpress timezone
        }

        //Country
        $country = '';
        $geo_info = $plugin->get_geo_info( $user_ip );
        if( $geo_info ){
            $country = $geo_info->country->name;
        }

        $user_ip = $plugin->get_masked_ip( $user_ip, false );
        $return .= "<tr class='cpp-$type' data-id='$log->ID'>";
        $return .= "<td>$consent_type</td>";
        $return .= "<td>$consent_status</td>";
        $return .= "<td>$accepted_all</td>";
        $return .= "<td>$accepted_cookies</td>";
        $return .= "<td>$declined_cookies</td>";
        $return .= "<td>$user_id</td>";
        $return .= "<td>$country</td>";
        $return .= "<td>$user_ip</td>";
        $return .= "<td>$platform</td>";
        $return .= "<td>$browser</td>";
        $return .= "<td>$desktop_or_mobile</td>";
        $return .= "<td>$page_url</td>";
        //        $return .= "<td>$cookie_name</td>";
        $return .= "<td>$expiration_time</td>";
        $return .= "<td>$date_created</td>";

        $return .= "<td><a href='#' class='ampp-delete-log'><i class='xbox-icon xbox-icon-trash xbox-color-red'></i></a></td>";
        $return .= "</tr>";
    }
    $return .= "</tbody></table>";
    return $return;
}


function cookieplus_render_overview_box( $type, $title = '', $count = 0, $src_image = '', $button = '' ){
    $html = '';
    $html .= "<div class='cpp-overview-box cpp-$type'>";
    $html .= "<div class='cpp-image'>";
    $html .= "<img src='$src_image'>";
    $html .= "</div>";
    $html .= "<div class='cpp-number'>";
    $html .= $count;
    $html .= "</div>";
    $html .= "<div class='cpp-title'>";
    $html .= $title;
    $html .= "</div>";
    $html .= $button;
    $html .= "</div>";
    return $html;
}

function get_ids_by_query_cookieplus( $query ){
    global $wpdb;
    $data = $wpdb->get_results( $query, 'ARRAY_A' );
    $results = array();
    foreach( $data as $entry ){
        $results[] = $entry['post_id'];
    }
    return $results;
}

function get_consent_logs_ids_cookieplus( $prefix, $args ){
    global $wpdb;
    $ids = array();

    //Consultar ids solo en el panel de opciones
    $post_type = \MasterPopups\Includes\Functions::get_current_post_type();
    if( ! in_array( $post_type, array( 'master-popups' ) ) ){
        return array(
            'declined' => array(),
            'terms' => array(),
            'privacy_policy' => array()
        );
    }

    foreach( $args as $type => $data ){
        $ids[$type] = get_ids_by_query_cookieplus( "
            SELECT post_id
            FROM {$wpdb->prefix}postmeta
            WHERE meta_key = '" . $data['key'] . "'
            AND meta_value = '" . $data['value'] . "'
        " );
    }
    return $ids;
}

$args = array(
    'declined' => array(
        'key' => $prefix . 'declined',
        'value' => true,
    ),
    'terms' => array(
        'key' => $prefix . 'type',
        'value' => CPMPP_KEY_TERMS_CONSENT,
    ),
    'privacy_policy' => array(
        'key' => $prefix . 'type',
        'value' => CPMPP_KEY_PRIVACY_CONSENT,
    )
);

$consent_logs_ids = get_consent_logs_ids_cookieplus( $prefix, $args );
$logs_cookies_declined = count( $consent_logs_ids['declined'] );
$logs_terms = count( $consent_logs_ids['terms'] );
$logs_privacy = count( $consent_logs_ids['privacy_policy'] );
$logs_cookies_accepted = $total_consent_logs - $logs_cookies_declined - $logs_terms - $logs_privacy;

$content = '';
$content .= "<div class='cpp-rows xbox-clearfix'>";
$button_scan_cookies = "<a href='javascript:void(0)' class='xbox-btn xbox-btn-teal cpp-btn-go-scan-cookies'>Scan Cookies</a>";
$content .= cookieplus_render_overview_box( 'consent-logs', 'Consent Logs', $total_consent_logs, $images_url . 'icon-consent-logs.png' );
$content .= cookieplus_render_overview_box( 'total-cookies', 'Total Cookies', count( $_COOKIE ), $images_url . 'icon-cookies.png', $button_scan_cookies );
$content .= cookieplus_render_overview_box( CPMPP_KEY_COOKIE_CONSENT, 'Cookies Consent Accepted', $logs_cookies_accepted, $images_url . 'icon-cookies-accepted.png' );
$content .= cookieplus_render_overview_box( CPMPP_KEY_COOKIE_CONSENT, 'Cookies Consent Rejected', $logs_cookies_declined, $images_url . 'icon-cookies-rejected.png' );
$content .= cookieplus_render_overview_box( CPMPP_KEY_TERMS_CONSENT, 'Terms & Conditions Accepted', $logs_terms, $images_url . 'icon-terms-conditions-accepted.png' );
$content .= cookieplus_render_overview_box( CPMPP_KEY_PRIVACY_CONSENT, 'Privacy Policy Accepted', $logs_privacy, $images_url . 'icon-privacy-policy-accepted.png' );
$content .= "</div><!--.cpp-row-->";


$xbox->add_field( array(
    'type' => 'title',
    'name' => __( 'CookiePlus Overview', 'mpp-cookieplus' ),
) );

$xbox->add_field( array(
    'id' => 'dashboard-overview',
    'type' => 'html',
    'options' => array(
        'show_name' => false,
    ),
    'content' => $content,
    'grid' => '8-of-8',
) );


$xbox->add_field( array(
    'type' => 'title',
    'name' => __( 'Consent Logs', 'mpp-cookieplus' ),
    'desc' => "Total: <span class='info-total-consent-logs'>$total_consent_logs</span>",
) );


$xbox->open_mixed_field( array(
    'id' => 'logs-count',
    'name' => __( 'Number of logs to load', 'mpp-cookieplus' ),
    'desc' => __( 'Enter the number of logs to load in the table. -1= All Logs', 'mpp-cookieplus' ),
) );
$xbox->add_field( array(
    'id' => 'consent-logs-number',
    'type' => 'number',
    'default' => self::$defaults['consent-logs-number'],
    'options' => array(
        'show_name' => false,
        'desc_tooltip' => true,
        'show_spinner' => true,
        'show_unit' => false,
    ),
    'attributes' => array(
        'min' => -1,
    ),
) );
$xbox->add_field( array(
    'id' => 'btn-load-consent-logs',
    'type' => 'button',
    'content' => __( 'Load Logs', 'mpp-cookieplus' ),
    'options' => array(
        'show_name' => false,
        'color' => 'purple',
        'size' => 'normal',
        'tag' => 'a',
        'icon' => '<i class="xbox-icon xbox-icon-refresh"></i>'
    ),
) );
$xbox->close_mixed_field();

$load_number_logs = Options::option( 'consent-logs-number' );

$xbox->add_field( array(
    'id' => 'dashboard-consent-logs',
    'type' => 'html',
    'options' => array(
        'show_name' => false,
    ),
    'content' => cookieplus_get_consent_logs_table( $load_number_logs ),
    'grid' => '8-of-8',
) );



//Mismo tiempo que la función usada
//function cookieplus_get_consent_logs_table2( $count = 100 ){
//    $return = '';
//    $plugin = Functions::get_plugin_instance();
//    $consent_logs = $plugin->get_consent_logs( array( 'numberposts' => $count, 'fields' => 'ids' ) );
//
//    if( Functions::is_empty( $consent_logs ) ){
//        return $return;
//    }
//
//    $return .= "<table id='cpp-table-cookie-logs-overview' class='ampp-table'>";
//    $return .= "<thead><tr>";
//    $return .= "<th class='noHide'>" . __( 'Consent Type', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th class='noHide'>" . __( 'Status', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Accepted All', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Accepted Cookies', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Declined Cookies', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'User ID', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Country', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'IP Address', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Platform', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Browser', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Device', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Page URL', 'mpp-cookieplus' ) . "</th>";
//    //    $return .= "<th>" . __( 'Cookie Name', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Expiration', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th><i class='xbox-icon xbox-icon-calendar'></i> " . __( 'Date Created', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th class='noHide'><input type='checkbox' class='ampp-checkbox-all-logs'><a href='#' class='ampp-delete-all-logs' title='Delete'><i class='xbox-icon xbox-icon-trash xbox-color-red'></i></a></th>";
//    $return .= "</tr></thead><tbody>";
//
//    foreach( $consent_logs as $log_id ){
//        $type = get_post_meta( $log_id, 'cpp_type', true );
//        $version = get_post_meta( $log_id, 'cpp_version', true );
//        $accepted = get_post_meta( $log_id, 'cpp_accepted', true );
//        $declined = get_post_meta( $log_id, 'cpp_declined', true );
//        //$cookie_name = get_post_meta( $log_id, 'cpp_cookie_name', true );
//        $expiration_time = get_post_meta( $log_id, 'cpp_expiration_time', true );
//        $date_created = get_post_meta( $log_id, 'cpp_date_created', true );
//        $groups = get_post_meta( $log_id, 'cpp_groups', true );
//        $groups_info = get_post_meta( $log_id, 'cpp_groups_info', true );
//        $user_id = get_post_meta( $log_id, 'cpp_user_id', true );
//        $user_ip = get_post_meta( $log_id, 'cpp_user_ip', true );
//        $user_agent = get_post_meta( $log_id, 'cpp_user_agent', true );
//        $browser = BrowserDetection::get_browser( $user_agent );
//        $platform = BrowserDetection::get_platform( $user_agent );
//        $desktop_or_mobile = BrowserDetection::is_mobile( $user_agent ) ? 'Mobile' : 'Desktop';
//        $page_url = get_post_meta( $log_id, 'cpp_page_url', true );
//
//        //Versión para: Privacy Policy & Terms & Conditions
//        $version_info = '';
//        if( $type == CPMPP_KEY_TERMS_CONSENT || $type == CPMPP_KEY_PRIVACY_CONSENT ){
//            $version_info = 'v' . $version;
//        }
//
//        //Consent Type
//        $accepted_text = $declined ? 'Declined' : 'Accepted';
//        $status_class = $accepted ? 'cpp-accepted' : 'cpp-declined';
//        $consent_status = "<span class='cpp-small cpp-small-bg $status_class'>$accepted_text $version_info</span>";
//        $title_array = array(
//            CPMPP_KEY_COOKIE_CONSENT => __( 'Cookies Consent', 'mpp-cookieplus' ),
//            CPMPP_KEY_TERMS_CONSENT => __( 'Terms & Conditions', 'mpp-cookieplus' ),
//            CPMPP_KEY_PRIVACY_CONSENT => __( 'Privacy Policy', 'mpp-cookieplus' ),
//        );
//        $consent_type = isset( $title_array[$type] ) ? $title_array[$type] : '';
//
//        //Consent info
//        $accepted_cookies = '';
//        $declined_cookies = '';
//        $accepted_all = ( $groups == 'all' || $groups == '' || $groups == null ) ? __( 'Yes', 'mpp-cookieplus' ) : __( 'No', 'mpp-cookieplus' );
//        if( $type == CPMPP_KEY_COOKIE_CONSENT ){
//            $groups_info = ! is_array( $groups_info ) ? array() : $groups_info;
//            foreach( $groups_info as $info ){
//                $status_class = $info['consent_given'] == 'true' ? 'cpp-accepted' : 'cpp-declined';
//                if( $info['consent_given'] == 'true' ){
//                    $accepted_cookies .= "<span class='cpp-small cpp-small-normal $status_class'>{$info['title']}</span>,<br>";
//                } else{
//                    $declined_cookies .= "<span class='cpp-small cpp-small-normal $status_class'>{$info['title']}</span>,<br>";
//                }
//            }
//            $accepted_cookies = rtrim( $accepted_cookies, ',<br>' );
//            $declined_cookies = rtrim( $declined_cookies, ',<br>' );
//        }
//
//        //Expiration
//        if( $expiration_time ){
//            $datetime = new DateTime( "@$expiration_time" );
//            $datetime->setTimeZone( Functions::get_timezone() );//wordpress timezone
//            $expiration_time = $datetime->format( "m/d/Y h:i:s a" );
//        }
//
//        //Date Created
//        if( $date_created ){
//            $date_created = date( "j M Y, g:i a", strtotime( $date_created ) );//date_created ya tiene wordpress timezone
//        }
//
//        //Country
//        $country = '';
//        $geo_info = $plugin->get_geo_info( $user_ip );
//        if( $geo_info ){
//            $country = $geo_info->country->name;
//        }
//
//        $user_ip = $plugin->get_masked_ip( $user_ip, false );
//        $return .= "<tr class='cpp-$type' data-id='$log_id'>";
//        $return .= "<td>$consent_type</td>";
//        $return .= "<td>$consent_status</td>";
//        $return .= "<td>$accepted_all</td>";
//        $return .= "<td>$accepted_cookies</td>";
//        $return .= "<td>$declined_cookies</td>";
//        $return .= "<td>$user_id</td>";
//        $return .= "<td>$country</td>";
//        $return .= "<td>$user_ip</td>";
//        $return .= "<td>$platform</td>";
//        $return .= "<td>$browser</td>";
//        $return .= "<td>$desktop_or_mobile</td>";
//        $return .= "<td>$page_url</td>";
//        //        $return .= "<td>$cookie_name</td>";
//        $return .= "<td>$expiration_time</td>";
//        $return .= "<td>$date_created</td>";
//
//        $return .= "<td><a href='#' class='ampp-delete-log'><i class='xbox-icon xbox-icon-trash xbox-color-red'></i></a></td>";
//        $return .= "</tr>";
//    }
//    $return .= "</tbody></table>";
//    return $return;
//}

//Mismo tiempo que la función usada
//function cookieplus_get_consent_logs_table3( $count = 100 ){
//    $return = '';
//    $plugin = Functions::get_plugin_instance();
//    global $wpdb;
//    $query = $wpdb->prepare( "SELECT pm.post_id, pm.meta_key, pm.meta_value FROM $wpdb->postmeta as pm WHERE meta_key LIKE %s ORDER BY post_id DESC", "cpp_%" );
//    $results = $wpdb->get_results( $query, ARRAY_A );
//
//    if( Functions::is_empty( $results ) ){
//        return $return;
//    }
//
//    $consent_logs = array();
//    foreach( (array) $results as $row ){
//        $post_id = $row['post_id'];
//        if( ! isset( $consent_logs[$post_id] ) ){
//            $consent_logs[$post_id] = array();
//        }
//        if( $row['meta_key'] == 'cpp_groups_info' ){
//            $consent_logs[$post_id][$row['meta_key']] = maybe_unserialize( $row['meta_value'] );
//        } else {
//            $consent_logs[$post_id][$row['meta_key']] = $row['meta_value'];
//        }
//
//        if( count( $consent_logs ) >= $count && count( $consent_logs[$post_id] ) >= 13 ){//porque son 13 campos que se guardan
//            break;
//        }
//    }
//
//    $return .= "<table id='cpp-table-cookie-logs-overview' class='ampp-table'>";
//    $return .= "<thead><tr>";
//    $return .= "<th class='noHide'>" . __( 'Consent Type', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th class='noHide'>" . __( 'Status', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Accepted All', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Accepted Cookies', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Declined Cookies', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'User ID', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Country', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'IP Address', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Platform', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Browser', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Device', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Page URL', 'mpp-cookieplus' ) . "</th>";
//    //    $return .= "<th>" . __( 'Cookie Name', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th>" . __( 'Expiration', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th><i class='xbox-icon xbox-icon-calendar'></i> " . __( 'Date Created', 'mpp-cookieplus' ) . "</th>";
//    $return .= "<th class='noHide'><input type='checkbox' class='ampp-checkbox-all-logs'><a href='#' class='ampp-delete-all-logs' title='Delete'><i class='xbox-icon xbox-icon-trash xbox-color-red'></i></a></th>";
//    $return .= "</tr></thead><tbody>";
//
//    foreach( $consent_logs as $log_id => $log_fields ){
//        $type = $log_fields['cpp_type'];
//        $version = $log_fields['cpp_version'];
//        $accepted = $log_fields['cpp_accepted'];
//        $declined = $log_fields['cpp_declined'];
//        //$cookie_name = isset( $log_fields['cpp_cookie_name'] ) ? : '';
//        $expiration_time = $log_fields['cpp_expiration_time'];
//        $date_created = $log_fields['cpp_date_created'];
//        $groups = $log_fields['cpp_groups'];
//        $groups_info = $log_fields['cpp_groups_info'];
//        $user_id = $log_fields['cpp_user_id'];
//        $user_ip = $log_fields['cpp_user_ip'];
//        $user_agent = $log_fields['cpp_user_agent'];
//        $browser = BrowserDetection::get_browser( $user_agent );
//        $platform = BrowserDetection::get_platform( $user_agent );
//        $desktop_or_mobile = BrowserDetection::is_mobile( $user_agent ) ? 'Mobile' : 'Desktop';
//        $page_url = $log_fields['cpp_page_url'];
//
//        //Versión para: Privacy Policy & Terms & Conditions
//        $version_info = '';
//        if( $type == CPMPP_KEY_TERMS_CONSENT || $type == CPMPP_KEY_PRIVACY_CONSENT ){
//            $version_info = 'v' . $version;
//        }
//
//        //Consent Type
//        $accepted_text = $declined ? 'Declined' : 'Accepted';
//        $status_class = $accepted ? 'cpp-accepted' : 'cpp-declined';
//        $consent_status = "<span class='cpp-small cpp-small-bg $status_class'>$accepted_text $version_info</span>";
//        $title_array = array(
//            CPMPP_KEY_COOKIE_CONSENT => __( 'Cookies Consent', 'mpp-cookieplus' ),
//            CPMPP_KEY_TERMS_CONSENT => __( 'Terms & Conditions', 'mpp-cookieplus' ),
//            CPMPP_KEY_PRIVACY_CONSENT => __( 'Privacy Policy', 'mpp-cookieplus' ),
//        );
//        $consent_type = isset( $title_array[$type] ) ? $title_array[$type] : '';
//
//        //Consent info
//        $accepted_cookies = '';
//        $declined_cookies = '';
//        $accepted_all = ( $groups == 'all' || $groups == '' || $groups == null ) ? __( 'Yes', 'mpp-cookieplus' ) : __( 'No', 'mpp-cookieplus' );
//        if( $type == CPMPP_KEY_COOKIE_CONSENT ){
//            $groups_info = ! is_array( $groups_info ) ? array() : $groups_info;
//            foreach( $groups_info as $info ){
//                $status_class = $info['consent_given'] == 'true' ? 'cpp-accepted' : 'cpp-declined';
//                if( $info['consent_given'] == 'true' ){
//                    $accepted_cookies .= "<span class='cpp-small cpp-small-normal $status_class'>{$info['title']}</span>,<br>";
//                } else{
//                    $declined_cookies .= "<span class='cpp-small cpp-small-normal $status_class'>{$info['title']}</span>,<br>";
//                }
//            }
//            $accepted_cookies = rtrim( $accepted_cookies, ',<br>' );
//            $declined_cookies = rtrim( $declined_cookies, ',<br>' );
//        }
//
//        //Expiration
//        if( $expiration_time ){
//            $datetime = new DateTime( "@$expiration_time" );
//            $datetime->setTimeZone( Functions::get_timezone() );//wordpress timezone
//            $expiration_time = $datetime->format( "m/d/Y h:i:s a" );
//        }
//
//        //Date Created
//        if( $date_created ){
//            $date_created = date( "j M Y, g:i a", strtotime( $date_created ) );//date_created ya tiene wordpress timezone
//        }
//
//        //Country
//        $country = '';
//        $geo_info = $plugin->get_geo_info( $user_ip );
//        if( $geo_info ){
//            $country = $geo_info->country->name;
//        }
//
//        $user_ip = $plugin->get_masked_ip( $user_ip, false );
//        $return .= "<tr class='cpp-$type' data-id='$log_id'>";
//        $return .= "<td>$consent_type</td>";
//        $return .= "<td>$consent_status</td>";
//        $return .= "<td>$accepted_all</td>";
//        $return .= "<td>$accepted_cookies</td>";
//        $return .= "<td>$declined_cookies</td>";
//        $return .= "<td>$user_id</td>";
//        $return .= "<td>$country</td>";
//        $return .= "<td>$user_ip</td>";
//        $return .= "<td>$platform</td>";
//        $return .= "<td>$browser</td>";
//        $return .= "<td>$desktop_or_mobile</td>";
//        $return .= "<td>$page_url</td>";
//        //        $return .= "<td>$cookie_name</td>";
//        $return .= "<td>$expiration_time</td>";
//        $return .= "<td>$date_created</td>";
//
//        $return .= "<td><a href='#' class='ampp-delete-log'><i class='xbox-icon xbox-icon-trash xbox-color-red'></i></a></td>";
//        $return .= "</tr>";
//    }
//    $return .= "</tbody></table>";
//    return $return;
//}