;(function ($) {
    //Copyright © 2018 Max Arthur López Pérez - All Rights Reserved.
    //Group Sorter v1.0
    function GroupSorter(element, options) {
        var _ = this;
        _.container = element;
        _.$container = $(element);
        _.values = {};
        var prefix = _.$container.attr('id') ? _.$container.attr('id') + '-' : '';
        _.defaults = {
            prefix: prefix || 'prefix-',
            showPlaceholder: true,
            classes: {
                container: "",
                group: "group",
                group_title: "group-title",
                list: "list",
                item: "item",
            },
        };

        _.options = $.extend(true, {}, _.defaults, options);
        _.init();
    }

    var proto = GroupSorter.prototype;

    proto.init = function () {
        var _ = this;
        _.$groups = _.$container.find('.groupsorter-group');
        _.$lists = _.$container.find('.groupsorter-list');
        _.$items = _.$container.find('.groupsorter-item');

        _.build();
        _.set_sortables();
        _.set_values();
    }

    proto.build = function () {
        var _ = this;
        _.$container.addClass('groupsorter ' + _.options.classes.container);
        _.$groups.addClass(_.options.classes.group);
        _.$lists.addClass(_.options.classes.list);
        _.$items.addClass(_.options.classes.item);

        _.$groups.each(function (i, group) {
            $(group).attr('id', _.prefixer(_.options.classes.group + '-' + (i + 1)));
            $(group).find('>.groupsorter-list').attr('id', _.prefixer(_.options.classes.list + '-' + (i + 1)));
        });

    }

    proto.prefixer = function (name) {
        var _ = this;
        return _.options.prefix + name;
    }

    proto.set_sortables = function () {
        var _ = this;
        _.$container.find('.groupsorter-list').sortable({
            delay: 50,
            forcePlaceholderSize: true,
            //containment: _.$container,
            connectWith: _.$container.find('.groupsorter-list'),
            sort: function (event, ui) {
                if (_.options.showPlaceholder) {
                    $(ui.item).parent().find('.ui-sortable-placeholder').css('visibility', 'visible');
                }
            },
            update: function (event, ui) {
                //Evita ejecución doble del evento cuando se trabaja con varias listas
                if (this === ui.item.parent()[0]) {
                    _.set_values();
                    var $sender = $(ui.sender);
                    _.$container.trigger('groupSorter_update_items', [_, _.values, ui, $sender]);
                }
            },
            receive: function (event, ui) {
                // c('On receive');
            },
        }).disableSelection();
    }

    proto.set_values = function () {
        var _ = this;
        _.$groups.each(function (i, group) {
            var values = [];
            $(group).find('.groupsorter-item').each(function (i, item) {
                values.push($(item).data('value'));
            });
            _.values[_.prefixer(_.options.classes.list + '-' + (i + 1))] = values;
        });
    }

    proto._get_values = function () {
        return this.values;
    }

    $.fn.groupSorter = function (arg) {
        var _ = this;
        var ret;
        _.each(function () {
            if (typeof arg === "object" || !arg) {
                this.groupSorter = new GroupSorter(this, arg);
            } else if (typeof arg === "string" && this.groupSorter && arg in this.groupSorter && arg.indexOf("_") === 0) {
                ret = this.groupSorter[arg]();
            }
        });
        return ret || _;
    };

    function c(msg) {
        console.log(msg);
    }

    function cc(msg, msg2) {
        console.log(msg, msg2);
    }

})(jQuery);